<?php
require_once('common/common.inc.php');
require_once('common/db_fns.inc.php');
require_once('common/misc_fns.inc.php');
require_once('libs/Smarty.class.php');
require_once('common/SException.inc.php');


class Parser {

	/**
	 * Constructor
	 */
	public function __construct() {

	}

	/**
	 * fetch attributes of the publication, parse it as string, use the propper templates (type1.tpl, type2.tpl, type3.tpl)
	 *
	 * @param array $attributes
	 * @param array $authors
	 * @param array $extra_attributes  extra attributes in format array(['name'] => 'value', ...)
	 * @param array $file_list  list of files in format for example array
	 * @return string  parsed text block
	 */
	public function get_parsed($format_id, $attributes, $authors, $extra_attributes=null, $file_list) {
		//remove html special chars.
		$attributes = htmlspecialchars_rec($attributes);
		$authors = htmlspecialchars_rec($authors);
		$format_id = intval($format_id);
		$extra_attributes = htmlspecialchars_rec($extra_attributes);

		$smarty = new smarty();
		//register the resource name "db"
		$smarty->register_resource("db", array("db_get_template", "db_get_timestamp", "db_get_secure", "db_get_trusted"));
		$smarty->assign('pub_type', $attributes['pub_type']);
		$smarty->assign('country', $attributes['country_code']?countries($attributes['country_code']):'');
		$smarty->assign('state', $attributes['state_code']?states($attributes['state_code']):'');
		$smarty->assign('city', $attributes['city']);
		$smarty->assign('publisher', $attributes['publisher']);
		$smarty->assign('language', $attributes['language']);
		$smarty->assign('title', $attributes['title']);
		$smarty->assign('isbn', $attributes['isbn']);
		$smarty->assign('issn', $attributes['isbn']);
		$smarty->assign('ixxn', strlen($attributes['isbn'])>9?'ISBN':'ISSN'); //by the workshop decite whether to show ISBN or ISSN
		$smarty->assign('pp', $attributes['pp']);
		$smarty->assign('authors', $authors);
		$smarty->assign('files', $file_list);
		if(is_array($extra_attributes)) foreach ($extra_attributes as $name => $value) {
			$smarty->assign($name, $value);
		}
		$files='';
		if (is_array($file_list)) foreach($file_list as $file) {
			$files .= '<a href="./storage/'.$attributes['id'].'/'.$file.'" target="_self">'.$file.'</a> ';
		}
		$smarty->assign('files', $files);
		if ($attributes['pub_type'] == 1) {
			$smarty->assign('year', $attributes['year']);
			//return $smarty->fetch('type1.tpl');
		}
		elseif ($attributes['pub_type'] == 2) {
			$smarty->assign('year', $attributes['year']);
			$smarty->assign('issue_date', $attributes['month_year']);
			$smarty->assign('vol', $attributes['vol']);
			$smarty->assign('no', $attributes['no']);
			//return $smarty->fetch('type2.tpl');
		}
		elseif ($attributes['pub_type'] == 3) {
			$smarty->assign('year', $attributes['year']);
			$smarty->assign('w_from', $attributes['w_from']=='0000-00-00'?'':$attributes['w_from']);
			$smarty->assign('w_to', $attributes['w_to']=='0000-00-00'?'':$attributes['w_to']);
			$smarty->assign('workshop_name', $attributes['workshop_name']);
			//return $smarty->fetch('type3.tpl');
		}
		return $smarty->fetch("db:$format_id");
	}
}



/*
	************ Smarty plugin *************
*/
	$db =& db_connect();

	function db_get_template($tpl_name, &$tpl_source, &$smarty_obj)
	{
		// do database call here to fetch your template,
		// populating $tpl_source
		global $db;
		//print_r($db);die;
		$res =& $db->query("SELECT content
							FROM `format`
							WHERE id='$tpl_name'");
		if (PEAR::isError($res)) {
			throw new SException($res->getMessage());
		}
		if ($res->numRows()) {
			$row = $res->fetchRow();
			$tpl_source = $row[0];
			return true;
		} else { //else select default template
			$res =& $db->query("SELECT content FROM `format` ORDER BY id");
			if (PEAR::isError($res)) {
				throw new SException($res->getMessage());
			}
			if ($res->numRows()) {
				$row = $res->fetchRow();
				$tpl_source = $row[0];
				return true;
			} else {
				return false;
			}
		}
	}

	function db_get_timestamp($tpl_name, &$tpl_timestamp, &$smarty_obj)
	{
		// do database call here to fetch your template,
		// populating $tpl_source
		global $db;
		$res =& $db->query("SELECT timestamp
							FROM `format`
							WHERE id='$tpl_name'");
		if (PEAR::isError($res)) {
			throw new SException($res->getMessage());
		}
		if ($res->numRows()) {
			$row = $res->fetchRow();
			$tpl_timestamp = $row[0];
			return true;
		} else { //else select default template
			$res =& $db->query("SELECT timestamp FROM `format` ORDER BY id");
			if (PEAR::isError($res)) {
				throw new SException($res->getMessage());
			}
			if ($res->numRows()) {
				$row = $res->fetchRow();
				$tpl_timestamp = $row[0];
				return true;
			} else {
				return false;
			}
		}
	}

	function db_get_secure($tpl_name, &$smarty_obj)
	{
		// assume all templates are secure
		return true;
	}

	function db_get_trusted($tpl_name, &$smarty_obj)
	{
		// not used for templates
	}


/*
	End of Smarty plugin
*/
?>