<?php
session_start();
require_once('../common/common.inc.php');
require_once('../libs/Smarty.class.php');
require_once('../common/db_fns.inc.php');
require_once('../common/misc_fns.inc.php');
require_once('../common/SException.inc.php');


/**
 * Generates 3 inquiry forms. Leads the submitter to add a new publication.
 *
 */
class Load {
	private $db;
	private $smarty;
	private $request;
	private $auth_user;
	private $auth_user_id;
	private $country_list;
	private $state_list;
	private $strict_mode;
	private $permitted_extensions;

	/*	Uses sessions:
		$_SESSION['feedback_st1'] - saved form data of load_st1.tpl template
		$_SESSION['feedback_st2'] - saved form data of load_st2.tpl template
		$_SESSION['feedback_st3'] - saved form data of load_st3.tpl template
		$_SESSION['tmp_attr'] - new extra attributes temporarily added by current user
		$_SESSION['tmp_cat'] - temporarily added categories
	*/

	/**
	 * Constructor
	 *
	 * @param array $request  usually $_POST
	 * @param string $auth_user  CTU common format username
	 * @param int $strict_mode  check form data strict or messily (values are 0 - 1)
	 * @param int $permitted_extensions  permitted file extensions
	 */
	public function __construct($request, $auth_user, $strict_mode, $permitted_extensions) {
		$this->permitted_extensions = $permitted_extensions;
		$this->strict_mode = $strict_mode;
		$this->request = $request;
		$this->country_list = countries();
		$this->state_list = states();
		$this->db = db_connect();
		$this->auth_user = $auth_user;
		$sql = "SELECT id FROM `submitter` WHERE ctu_login = '".$auth_user."'";
		$data =& $this->db->getOne($sql);
		if (PEAR::isError($data)) {
		    die("User doesn't exist!");
		}
		if (!$data) throw new SException("Uživatel $auth_user neexistuje");
		$this->auth_user_id =& $data;
		$this->smarty = new Smarty;
		$this->smarty->config_dir = '../configs/';
		$this->smarty->template_dir = '../templates/';
		$this->smarty->compile_dir = '../templates_c/';
		//$this->smarty->compile_check = true;
		//$this->smarty->debugging = true;
	}

	/**
	 * Execute page, the only entry point of Load class
	 *
	 */
	public function execute() {
		$request = $this->request;
		if ($request['delete_author']) {
			try {
				$_SESSION['feedback_st1'] = $this->request; //save typed form data
			 	$a = $request['authors'];
				if (is_array($a)) {
					$affected = 0;
					foreach ($a as $i => $value)
						if (!$affected) $affected = $this->delete_author($value);
						else $this->delete_author($value);
				} else {
					$affected = $this->delete_author($a);
				}
				if ($affected)
					$this->smarty->assign("operation_success", "Autoři byli smazáni");
				else {
					$this->smarty->assign("operation_error", 'Nemáte práva mazat tento záznam');
				}
				$this->display_st1();
			} catch (Exception $e) {
				$this->smarty->assign("operation_error", $e->getMessage());
				$this->display_st1();
			}
		}
		else if ($request['delete_publisher']) {
			try {
				$_SESSION['feedback_st2'] = $this->request; //save typed form data
				$affected = $this->delete_publisher($request['publisher']);
				if ($affected)
					$this->smarty->assign("operation_success", "Vydavatel byl smazán");
				else {
					$this->smarty->assign("operation_error", 'Nemáte práva mazat tento záznam');
				}
				$this->display_st2();
			} catch (Exception $e) {
				$this->smarty->assign("operation_error", $e->getMessage());
				$this->display_st2();
			}
		}
		else if ($request['add_author']) {
			try {
				$_SESSION['feedback_st1'] = $this->request; //save typed form data
				$this->add_author($request['author_name'], $request['author_middle_name'], $request['new_author_surname']);
				$this->smarty->assign("operation_success", "Autor byl přidán");
				$this->display_st1();
			} catch (SException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign("operation_error", $e->getMessage());
				$this->display_st1();
			}
		}
		else if ($request['delete_added_categories']) {
			try {
				$_SESSION['feedback_st1'] = $this->request; //save typed form data
				unset($_SESSION['tmp_cat']);
				$this->smarty->assign("operation_success", "Přidané kategorie smazány");
				$this->display_st1();
			} catch (SException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign("operation_error", $e->getMessage());
				$this->display_st1();
			}
		} else if ($request['add_publisher']) {
			try {
				$_SESSION['feedback_st2'] = $this->request; //save typed form data
				$this->add_publisher($request['publisher_name']);
				$this->smarty->assign('operation_success', 'Vydavatel byl přidán');
				$this->display_st2();
			} catch (SException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
				$this->display_st2();
			}
		}
		else if ($request['add_category']) {
			try {
				$_SESSION['feedback_st1'] = $this->request; //save typed form data
				$this->add_tmp_category($request['category_name']);
				$this->smarty->assign('operation_success', 'Kategorie byla přidána');
				$this->display_st1();
			} catch (SException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
				$this->display_st1();
			}
		} else if ($request['add_attribute']) {
			$_SESSION['feedback_st2'] = $this->request; //save typed form data
			try {
				$this->validate_add_att();
				$this->add_tmp_attribute($request['new_attribute_name'], $request['new_attribute_description'], $request['new_attribute_value']);
				$this->display_st2();
			} catch (SException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
				$this->display_st2();
			}
		}
		else if ($request['delete_attributes']) {
			$_SESSION['feedback_st2'] = $this->request; //save typed form data
			try {
				$this->delete_attributes();
				$this->display_st2();
			} catch (SException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				die($e->getMessage());
			}
		}
		else if ($request['previous']) {
			switch ($request['prev_stage']) {
				case(1):
					$_SESSION['feedback_st2'] = $this->request; //save typed form data
					$this->display_st1();
					break;
				case(2):
					$this->display_st2();
					break;
				case(3):
					$this->display_st3();
					break;
				default:
					$this->display_st1();
			}
		}
		else if (!$request || $request['next']) {
			switch ($request['next_stage']) {
				case(''): //start stage
					$this->display_st1();
					break;
				case(2): //additional attributes
					try {
						$this->validate_st1();
						$this->display_st2();
					} catch (SException $s) {
						die($s->getMessage());
					} catch (Exception $e) {
						$this->smarty->assign("operation_error", $e->getMessage());
						$this->display_st1();
					}
					break;
				case(3): //confirm stage
					try {
						$this->validate_st2();
						$this->display_st3();
					} catch (SException $s) {
						die($s->getMessage());
					} catch(Exception $e) {
						$this->smarty->assign("operation_error", $e->getMessage());
						$this->display_st2();
					}
					break;
				case(4): //finish stage
					try {
						$this->validate_st3();
						$this->finish();
					} catch (SException $s) {
						die($s->getMessage());
					} catch(Exception $e) {
						$this->smarty->assign("operation_error", $e->getMessage());
						$this->display_st3();
					}
					break;
				default:
					$this->display_st1();
			}
		}
		else {
       		$this->display_st1();
	    }
	}

	/**
	 * Delete unconfirmed and unbinded author previously inserted by the current user
	 *
	 * @param int $id
	 * @return int count of affected rows
	 */
	private function delete_author($id) {
		$sql = "DELETE FROM `author`
				WHERE id = '$id' AND confirmed = '0' AND submitter_id = '".$this->auth_user_id."'
				AND '$id' NOT IN
				(SELECT DISTINCT author_id FROM `author_has_publication`)";
		$res = $this->db->query($sql);
		unset($_SESSION['feedback_st1']['authors']);
		if (PEAR::isError($res)) throw new Exception($res->getMessage(), 1);
		return $this->db->affectedRows();
	}

	/**
	 * Delete unconfirmed and unbinded publisher, previously inserted by the current user
	 *
	 * @param int $id publisher id
	 * @return int count of affected rows
	 */
    private function delete_publisher($id) {
		$sql = "DELETE FROM `publisher`
				WHERE id = '$id' AND confirmed = '0' AND submitter_id = '".$this->auth_user_id."'
				AND '$id' NOT IN
				(SELECT DISTINCT publisher_id FROM `publication`)";
		$res = $this->db->query($sql);
		unset($_SESSION['feedback_st1']['publisher']);
		if (PEAR::isError($res)) throw new Exception($res->getMessage());
		return $this->db->affectedRows();
	}

	/**
	 * Delete temporary attribute from $_SESSION
	 *
	 *
	 */
	private function delete_attributes() {
		unset($_SESSION['tmp_attr']);
	}

	/**
	 * Add new author
	 *
	 * @param string $name
	 * @param string $middlename
	 * @param string $surname
	 */
	private function add_author($name, $middlename, $surname) {
		if (sizeof($name) > 20 || !$name) throw new Exception('Délka jména je min. 1 a max. 20 znaků');
		if (sizeof($middlename) > 20) throw new Exception('Délka prostředního jména je max. 20 znaků');
		if (sizeof($surname) > 20 || !$surname) throw new Exception('Délka příjmení je min. 1 a max. 20 znaků');
		if ($this->db->getOne("SELECT id FROM `author` WHERE name='$name' AND middlename='$middlename' AND surname='$surname'
								AND (submitter_id='".$this->auth_user_id."' OR confirmed=1)"))
			throw new Exception("Vkládaný autor již existuje".$ppp);
		$sql = "INSERT INTO `author` VALUES ('', '".$this->auth_user_id."', '$name', '$surname', '$middlename', '0')";
		$res = $this->db->query($sql);
		if (PEAR::isError($res)) throw new SException($res->getMessage());
	}

	/**
	 * Add new publisher
	 *
	 * @param string $name  name of the new publisher
	 */
	private function add_publisher($name) {
		if (!$name)
        	throw new Exception('Chybná informace o novém vydavateli!');
		if (sizeof($name) > 45)
        	throw new Exception('Maximální délka jména vydavatele je 45 znaků');
		if ($this->db->getOne("SELECT id FROM `publisher` WHERE name='$name'
								AND (submitter_id='".$this->auth_user_id."' OR confirmed=1)"))
			throw new Exception('Vydavatel již existuje');
		$sql = "INSERT INTO `publisher` VALUES ('', '".$this->auth_user_id."', '$name', '0')";
		$res = $this->db->query($sql);
		if (PEAR::isError($res)) throw new SException($res->getMessage());
	}

	/**
	 * Creates a temporary information about new attributes
	 *
	 * @param string $name  name of the new added attribute
	 * @param string $description  description of the new added attribute
	 * @param string $value  text value of the attribute
	 */
	private function add_tmp_attribute($name, $description, $value) {
		$_SESSION['tmp_attr']['name'][$name] = $value;
		$_SESSION['tmp_attr']['description'][$name] = $description;
	}

	/**
	 * Creates a temporary information about new category
	 *
	 * @param string $name  name of the new added attribute
	 * @param string $description  description of the new added attribute
	 * @param string $value  text value of the attribute
	 */
	private function add_tmp_category($name) {
		if (sizeof($name) > 255)
			throw new Exception('Jméno kategorie je příliš dlouhé (max 40 znaků)');
		elseif (!$name)
			throw new Exception('Nezadal jste jméno kategorie');
		if ($this->db->getOne("SELECT id FROM `categories` WHERE name='$name'
								AND (submitter_id='".$this->auth_user_id."' OR confirmed=1)"))
			throw new Exception('Kategorie se jménem '.$name.' již existuje');
		if (!is_array($_SESSION['tmp_cat']))
			$_SESSION['tmp_cat'][] = $name;
		else
			if (!in_array($name, $_SESSION['tmp_cat'])) $_SESSION['tmp_cat'][] = $name;
	}

	/**
	 * Add new attribute to the publication, assign a value to it
	 *
	 * @param int $publication_id  id of the publication to which this attribute is beeing assigned
	 * @param string $name  name of the new added attribute
	 * @param string $description  the name of the attribute
	 * @param string $value  text value of the attribute
	 */
	private function add_attribute($publication_id, $name, $description, $value) {
		if (!$description || !$value || $description=="jméno nového atributu" || $value=="hodnota atributu")
			throw new Exception('Špatně vyplněné údaje u nového atributu');
		$this->db->autoCommit(false);
		$submitter_id = $this->auth_user_id;
		$sql = "INSERT INTO `attributes` VALUES ('', '$submitter_id', '$name', '$description', '0')";
		$res1 =& $this->db->query($sql);
		if (DB::isError($res1)) {
			$this->db->rollBack();
			throw new Exception($res1->getMessage());
		}
		$attributes_id = last_insert_id($this->db);
		$sql = "INSERT INTO `attrib_storage` VALUES ('$publication_id', '$attributes_id', '$submitter_id', '$value')";
		$res2 = $this->db->query($sql);
		if (DB::isError($res2)) {
			$this->db->rollBack();
			throw new SException($res2->getMessage());
		}
		$this->db->commit();
	}

	/**
	 * Display the 1. stage of the forms
	 *
	 */
	private function display_st1() {
		$sql = "SELECT id, CONCAT_WS(' ',name,middlename,surname) AS author
				FROM `author`
                WHERE confirmed = '1' OR submitter_id = '".$this->auth_user_id."'
				ORDER BY surname";
		$this->smarty->assign('authors', htmlspecialchars_rec($this->db->getAssoc($sql)));
		$sql = "SELECT id, name
				FROM `categories`
				WHERE confirmed='1' OR submitter_id = '".$this->auth_user_id."'
				ORDER BY name";
		$this->smarty->assign('categories', htmlspecialchars_rec($this->db->getAssoc($sql)));
		//take confirmed results, or results owned by current user
		$this->smarty->assign("strict_mode", $this->strict_mode);
		$this->smarty->assign("logged_info", $this->auth_user);
		$this->smarty->assign("type_no", array(1, 2, 3));
		$this->smarty->assign("type_names", array("Kniha", "Časopis", "Konference"));
		$this->smarty->assign("lang_no", array(1,2,3));
		$this->smarty->assign("langs", array("Czech", "English", "German"));
		$this->smarty->assign("country", $this->country_list);
		$this->smarty->assign("state", $this->state_list);
		$s=null;
		if (is_array($_SESSION['tmp_cat'])) foreach($_SESSION['tmp_cat'] as $name) {
			$s[] = $name;
		}
		$this->smarty->assign('added_categories', stripslashes_rec(htmlspecialchars_rec($s)));
		if (isset($_SESSION['feedback_st1'])) $this->feedback_st1(); //feedback
		$this->smarty->display('load_st1.tpl');
	}

	/**
	 * Display the 2. stage of the forms
	 *
	 */
	private function display_st2() {
		if (isset($_SESSION['feedback_st2'])) $this->feedback_st2(); //feedback
		$sql = "SELECT id, name
				FROM `publisher`
				WHERE confirmed='1' OR submitter_id = '".$this->auth_user_id."'
				ORDER BY name";
		$this->smarty->assign('publisher', htmlspecialchars_rec($this->db->getAssoc($sql)));
		$this->smarty->assign("strict_mode", $this->strict_mode);
		$this->smarty->assign("logged_info", $this->auth_user);
		$this->smarty->assign("decision_pub_type", $_SESSION['feedback_st1']['pub_type']);
		$sql = "SELECT id, name FROM attributes WHERE confirmed = '1'";
		$arr =& $this->db->getAssoc($sql);
		$sql = "SELECT id, description FROM attributes WHERE confirmed = '1'";
		$desc =& $this->db->getAssoc($sql);
		$out = '';
		foreach ($arr as $i => $v) {
			$out .= '<tr><td>'.$v.($desc[$i]?'<br />('.$desc[$i].')':'').'</td><td><input type="text" name="'.$v.'" size="20" value="'.stripslashes(htmlspecialchars($_SESSION['feedback_st2'][$v])).'" /></td></tr>'; //this row includes feedback operation
		}
		$this->smarty->assign("attributes", $out);	//insert form fields with additional attributes
		$s = '';
		$tmp = $_SESSION['tmp_attr']['name']; //$tmp is an array
		if (is_array($tmp)) {
			foreach ($tmp as $i => $value) {
				$description = $_SESSION['tmp_attr']['description'][$i];
				$s .= '<tr><td colspan="2">přidáváte atributy:</td></tr>
				<tr><td><strong>'.$i.'</strong>'.($description?'<br />('.stripslashes(htmlspecialchars($description)).')':'').'</td><td><span class="tmp_att_value">'.stripslashes(htmlspecialchars($value)).'</span></td></tr>';
			}
		}
		$this->smarty->assign('tmp_attr', $s);
		$this->smarty->display('load_st2.tpl');
	}

	/**
	 * Display the 3. stage of the forms
	 *
	 */
	private function display_st3() {
		$data = array_merge($_SESSION['feedback_st1'], $_SESSION['feedback_st2']);
		$data = stripslashes_rec(htmlspecialchars_rec($data));
		$authors = $data['authors'];
		$condition = ''; $i = 0;
		if (is_array($authors)) {
			$count = count($authors);
			foreach ($authors as $id) {
				if (++$i < $count) $condition .= "id = '$id' OR ";
				else $condition .= "id = '$id'";
			}
		} else {
			$condition = "id = '$authors'";
		}
		$sql = "SELECT id, CONCAT_WS(' ',name,middlename,surname) AS author
				FROM `author`
                WHERE (confirmed = '1' OR submitter_id = '".$this->auth_user_id."') AND ($condition)
				ORDER BY id";
		$sql3 = "SELECT name
				FROM `publisher`
				WHERE (confirmed = '1' OR submitter_id = '".$this->auth_user_id."') AND id = '".$data['publisher']."'";

		$author_names = $this->db->getAssoc($sql);
		$count = count($author_names); $i = 0; $authors_tog = '';
		foreach ($author_names as $id) {
			if (++$i < $count) $authors_tog .= $id.', ';
			else $authors_tog .= $id;
		}
		$this->smarty->assign("logged_info", $this->auth_user);
		$this->smarty->assign('review', $review);
		$this->smarty->assign('pub_type', $data['pub_type']);
		$this->smarty->assign('pub_lang', $data['pub_lang']);
		$this->smarty->assign('country', $data['country']?countries($data['country']):'');
		$this->smarty->assign('state', ($data['state']?states($data['state']):''));
		$this->smarty->assign('pub_title', $data['pub_title']);
        $this->smarty->assign('city', $data['city']);
        $this->smarty->assign('authors', htmlspecialchars_rec($authors_tog)); //associative authors
        $this->smarty->assign('publisher', stripslashes(htmlspecialchars($this->db->getOne($sql3)))); //publisher name
		$this->smarty->assign('isbn', $data['isbn']);
        $this->smarty->assign('issn', $data['issn']);
        $this->smarty->assign('pp', $data['pp']);
        $this->smarty->assign('issue_date', $data['issue_date']);
        $this->smarty->assign('vol', $data['vol']);
        $this->smarty->assign('no', $data['no']);
		$this->smarty->assign('workshop_name', $data['workshop_name']);
		$this->smarty->assign('execution_date', ($w_from && $w_to)?($data['w_from']." -> ".$data['w_to']):'');
		//checked categories
		$s = null;
		$propagated_categories = $_SESSION['feedback_st1']['propagated_categories'];
		if (is_array($propagated_categories))
			foreach ($propagated_categories as $i => $v) {
				$s[] = $this->db->getOne("SELECT name FROM `categories` WHERE id='$v'");
		}
		$this->smarty->assign('categories', stripslashes_rec(htmlspecialchars_rec($s)));
		//added categories
		$s = null; $tmp = $_SESSION['tmp_cat'];
		if (is_array($tmp)) {
			foreach ($tmp as $i => $v) {
				$s[] = $v;
			}
		}
		$this->smarty->assign('tmp_cat', stripslashes_rec(htmlspecialchars_rec($s)));
		//filled bublic attributes
		$s = null;
		$ex_attr = $this->filled_ext_attr();
		if (is_array($ex_attr))
			foreach ($ex_attr as $name) {
				$s .= stripslashes(htmlspecialchars($name)).': '.stripslashes(htmlspecialchars($_SESSION['feedback_st2'][$name])).'<br />';
		}
		$this->smarty->assign('attributes', $s);
		//added tmp attributes
		$s = null; $tmp = $_SESSION['tmp_attr']['name']; //$tmp is an array
		if (is_array($tmp)) {
			foreach ($tmp as $i => $v) {
				$s .= stripslashes(htmlspecialchars($i)).': '.stripslashes(htmlspecialchars($v)).'<br />';
			}
		}
		$this->smarty->assign('tmp_attr', $s);
		$out='';
		$permitted_extensions = $this->permitted_extensions;
		$count=count($permitted_extensions)-1;
		if ($permitted_extensions) foreach($permitted_extensions as $v) {
			$out .= $v.($count--?', ':''); //insert commas only between numbers
		}
		$this->smarty->assign('permitted_extensions', $out);
		$this->smarty->display('load_st3.tpl');
	}

	/**
	 * Gives written data back to form stage 1
	 *
	 */
	private function feedback_st1() {
		$feedback = $_SESSION['feedback_st1'];
		$feedback = stripslashes_rec($feedback);
		$this->smarty->assign('selected_pub_type', $feedback['pub_type']);
		$this->smarty->assign('selected_pub_lang', $feedback['pub_lang']);
		$this->smarty->assign('selected_country', $feedback['country']);
		$this->smarty->assign('selected_state', $feedback['state']);
		$this->smarty->assign('pub_title', $feedback['pub_title']);
		$this->smarty->assign('city', $feedback['city']);
		$this->smarty->assign('selected_authors', $feedback['authors']);
		$this->smarty->assign('selected_categories', $feedback['propagated_categories']);
	}

	/**
	 * Gives written data back to form stage 2
	 *
	 */
    private function feedback_st2() {
		//	foreach ($_SESSION['feedback_st2'] as $i => $v)
		//	$this->smarty->assign($i, $v)
		$feedback = $_SESSION['feedback_st2'];
		$feedback = htmlspecialchars_rec(stripslashes_rec($feedback));
        $this->smarty->assign('isbn', $feedback['isbn']);
        $this->smarty->assign('issn', $feedback['issn']);
        $this->smarty->assign('pp', $feedback['pp']);
        $this->smarty->assign('issue_date', $feedback['issue_date']);
        $this->smarty->assign('vol', $feedback['vol']);
        $this->smarty->assign('no', $feedback['no']);
        $this->smarty->assign('w_from', $feedback['w_from']);
        $this->smarty->assign('w_to', $feedback['w_to']);
		$this->smarty->assign('workshop_name', $feedback['workshop_name']);
		$this->smarty->assign('selected_publisher', $feedback['publisher']);
		$this->smarty->assign('new_attribute_name', $feedback['new_attribute_name']);
		$this->smarty->assign('new_attribute_value', $feedback['new_attribute_value']);
		$this->smarty->assign('new_attribute_description', $feedback['new_attribute_description']);

	}

	/**
	 * Save form 1 data to $_SESSION, check if typed data were valid
	 *
	 */
	private function validate_st1() {
		$_SESSION['feedback_st1'] = $this->request; //save typed form data
		extract($this->request);
		if (!($pub_type==1 || $pub_type==2 || $pub_type==3))
			throw new Exception('Není vybrán typ publikace');
		if (!($pub_lang==1 || $pub_lang==2 || $pub_lang==3))
			throw new Exception('Není vybrán jazyk');
		if (!$pub_title)
			throw new Exception('Chybí název publikace');
		if (!is_array($authors))
			throw new Exception('Nejsou vybráni autoři');
		if ($this->strict_mode) {
			if (!$city)
				throw new Exception('Chybí město');
			if (!($country && countries($country)))
				throw new Exception('Není vybrána země');
			if (($country=='US' || $country=='CA') && !($state && states($state)))
				throw new Exception('Není vybrán stát');
			if ($country!='US' && $country!='CA')
				unset($_SESSION['feedback_st1']['state']);
		}
	}

	/**
	 * Save form 2 data to $_SESSION and check if typed data are valid
	 *
	 */
	private function validate_st2() {
		$_SESSION['feedback_st2'] = $this->request; //save typed form data
		extract($this->request);
		if ($this->strict_mode) {
		//strict mode
			switch ($_SESSION['feedback_st1']['pub_type']) {
				case(1): //Book
					if (!(ereg('^[0-9]+-[0-9]+-[0-9]+-([0-9]+|X)$', $isbn)&&(strlen($isbn)==13) || ereg('^[0-9]+-[0-9]+-[0-9]+-([0-9]+|X)$', $isbn)&&(strlen($isbn)==17)))
						throw new Exception('Formát ISBN je neplatný');
					if (!ereg('^[0-9]{4}$', $issue_date))
						throw new Exception('Chybný formát datumu, zadejte YYYY');
					if (!ereg('^[0-9]{1,5}$', $pp))
						throw new Exception('Počet stran musí být číselněđ');
					break;
				case(2): //magazine
					if (!ereg('^[0-9]{4}-([0-9]{4}|[0-9]{3}X)$', $issn))
						throw new Exception('Formát ISSN je neplatný');
					if (!ereg('^[0-9]{4}-((0[1-9])|(1[0-2]))$', $issue_date))
						throw new Exception('Chybný formát datumu, zadejte YYYY-MM');
					if (!ereg('^[0-9]{1,5}-[0-9]{1,5}$', $pp))
						throw new Exception('Rozsah stránek je špatně, (zadejte např. 16-24)');
					if (!is_numeric($vol))
						throw new Exception('Položka vol. musí být číslo');
					if (!is_numeric($no))
						throw new Exception('Položka no. musí být číslo');
					break;
				case(3): //workshop
					if (!(ereg('^[0-9]+-[0-9]+-[0-9]+-([0-9]+|X)$', $isbn)&&(strlen($isbn)==13) || ereg('^[0-9]+-[0-9]+-[0-9]+-([0-9]+|X)$', $isbn)&&(strlen($isbn)==17)) && !ereg('^[0-9]{4}-([0-9]{4}|[0-9]{3}X)$', $isbn))
						throw new Exception('Formát ISBN/ISSN je neplatný');
					if (!ereg('^[0-9]{4}$', $issue_date))
						throw new Exception('Chybný formát datumu, zadejte YYYY');
					if (!$workshop_name)
						throw new Exception('Chybí jméno konference');
					if (!ereg('^[0-9]{4}-((0[1-9])|(1[0-2]))-((0[1-9])|(1[0-9])|(2[0-9])|(3[0-1]))$', $w_from))
						throw new Exception('Chybný formát datumu zahájení konference, zadejte YYYY-MM-DD');
					if (!ereg('^[0-9]{4}-((0[1-9])|(1[0-2]))-((0[1-9])|(1[0-9])|(2[0-9])|(3[0-1]))$', $w_to))
						throw new Exception('Chybný formát datumu ukončení konference, zadejte YYYY-MM-DD');
					break;
				default:
					throw new SException('Interní chyba, neplatný typ publikace');
			}
		} else {
			//weak controll mode
			switch ($_SESSION['feedback_st1']['pub_type']) {
				case(1): //Book
					if ($isbn && (!(ereg('^[0-9]+-[0-9]+-[0-9]+-([0-9]+|X)$', $isbn)&&(strlen($isbn)==13) || ereg('^[0-9]+-[0-9]+-[0-9]+-([0-9]+|X)$', $isbn)&&(strlen($isbn)==17))))
						throw new Exception('Formát ISBN je neplatný');
					if ($issue_date && !ereg('^[0-9]{4}$', $issue_date))
						throw new Exception('Chybný formát datumu, zadejte YYYY');
					if ($pp && !ereg('^[0-9]{1,5}$', $pp))
						throw new Exception('Počet stran musí být číselně');
					break;
				case(2): //magazine
					if ($issn && !ereg('^[0-9]{4}-([0-9]{4}|[0-9]{3}X)$', $issn))
						throw new Exception('Formát ISSN je neplatný');
					if ($issue_date && !ereg('^[0-9]{4}-((0[1-9])|(1[0-2]))$', $issue_date))
						throw new Exception('Chybný formát datumu, zadejte YYYY-MM');
					if ($pp && !ereg('^[0-9]{1,5}-[0-9]{1,5}$', $pp))
						throw new Exception('Rozsah stránek je špatně, (zadejte např. 16-24)');
					if ($vol && !is_numeric($vol))
						throw new Exception('Položka vol. musí být číslo');
					if ($no && !is_numeric($no))
						throw new Exception('Položka no. musí být číslo');
					break;
				case(3): //workshop
					if ($isbn && !(ereg('^[0-9]+-[0-9]+-[0-9]+-([0-9]+|X)$', $isbn)&&(strlen($isbn)==13) || ereg('^[0-9]+-[0-9]+-[0-9]+-([0-9]+|X)$', $isbn)&&(strlen($isbn)==17)) && !ereg('^[0-9]{4}-([0-9]{4}|[0-9]{3}X)$', $isbn))
						throw new Exception('Formát ISBN/ISSN je neplatný');
					if ($issue_date && !ereg('^[0-9]{4}$', $issue_date))
						throw new Exception('Chybný formát datumu, zadejte YYYY');
					if ($w_from && !ereg('^[0-9]{4}-((0[1-9])|(1[0-2]))-((0[1-9])|(1[0-9])|(2[0-9])|(3[0-1]))$', $w_from))
						throw new Exception('Chybný formát datumu zahájení konference, zadejte YYYY-MM-DD');
					if ($w_to && !ereg('^[0-9]{4}-((0[1-9])|(1[0-2]))-((0[1-9])|(1[0-9])|(2[0-9])|(3[0-1]))$', $w_to))
						throw new Exception('Chybný formát datumu ukončení konference, zadejte YYYY-MM-DD');
					break;
				default:
					throw new SException('Interní chyba, neplatný typ publikace');
			}
		}
	}

	/**
	 * Checks whether submitter has sent a pdf/ppt or not
	 *
	 */
	private function validate_st3() {
		$permitted_extensions = $this->permitted_extensions;
		foreach($_FILES['documents']["error"] as $key => $error) {
			if ($error == UPLOAD_ERR_OK) {
				$file =& $_FILES['documents']['tmp_name'][$key];
				$name =& $_FILES['documents']['name'][$key];
				$extension = substr($name, strrpos($name, ".") + 1);
				if ($permitted_extensions) {
					if (!in_array($extension, $permitted_extensions)) throw new Exception("Vkládáte soubor s nepovolenou příponou.");
				}
				if ((!is_uploaded_file($file) && $file))
					throw new SException('Not an uploaded file.');
			}
		}
	}

	/**
	 * Validate temporarily added extensible attributes
	 *
	 */
	private function validate_add_att(){
		$_SESSION['feedback_st2'] = $this->request; //save typed form data
		extract($this->request);
		if (!ereg('^_([0-9]|[a-zA-Z]){0,31}$', $new_attribute_name))
			throw new Exception('První znak jména vkládaného atributu musí být podtržítko, jméno může obsahovat písmena anglické abecedy a číslice, max. však 32 znaků.');
		if (!$new_attribute_value)
			throw new Exception('Chybí hodnota pro nově vkládaný atributut');
		if (sizeof($new_attribute_description) > 255)
			throw new Exception('Popisek nového atributu je příliš dlouhý');
		unset($_SESSION['feedback_st2']['new_attribute_name']);
		unset($_SESSION['feedback_st2']['new_attribute_value']);
		unset($_SESSION['feedback_st2']['new_attribute_description']);
	}

	/**
	 * Recognize which extensible attribute in form stage 2 have been assigned
	 *
	 * @return array of $attribute_id => $attrinute_name
	 */
	private function filled_ext_attr() {
		foreach ($_SESSION['feedback_st2'] as $name => $value) {
			$sth = $this->db->prepare("SELECT id, name FROM `attributes` WHERE name=? AND confirmed='1'");
			if (PEAR::isError($sth)) {
				$this->db->rollBack();
				throw new SException($sth->getMessage());
			}
			$res =& $this->db->execute($sth, $name);
			if ($res->numRows()) {
				$row = $res->fetchRow();
				$id = $row[0]; $nm = $row[1];
				if ($_SESSION['feedback_st2'][$nm] != '') //if value of the extensible attribute in form stage 2 had been assigned
					$public_attr[$id] = $nm;
			}
		}
		return $public_attr;
	}

	/**
	 * Finish adding new publication, save pdf/ppt
	 *
	 */
	private function finish() {
		$submitter_id = $this->auth_user_id;
		$data = array_merge($_SESSION['feedback_st1'], $_SESSION['feedback_st2']);
		$publisher_id = $data['publisher'];
		$country_code = $data['country'];
		$state_code = $data['state'];
		$city = $data['city'];
		$pub_type = $data['pub_type'];
		$language = $data['pub_lang'];
		$title = $data['pub_title'];
		$authors = $data['authors']; //an array
		if ($pub_type == 1) {
			$isbn = $data['isbn'];
			$issue_date = $data['issue_date'].'-01-01';
			$pp = $data['pp'];
		} elseif ($pub_type == 2) {
			$isbn = $data['issn'];
			$issue_date = $data['issue_date'].'-01';
			$vol = $data['vol'];
			$no = $data['no'];
			$pp = $data['pp'];
		} elseif ($pub_type == 3) {
			$isbn = $data['isbn'];
			$issue_date = $data['issue_date'].'-01-01';
			$workshop_name = $data['workshop_name'];
			$w_from = $data['w_from'];
			$w_to = $data['w_to'];
			$pp = $data['pp'];
		}
		$this->db->autoCommit(false);
		//insert into `publication`
		$sql = "INSERT INTO `publication` VALUES
				('', '$submitter_id', '$publisher_id', '$country_code', '$state_code', '$city', '$pub_type', '$language', '$title', '0', '$issue_date', '$w_from', '$w_to', '$workshop_name', '$isbn', '$vol', '$no', '$pp')";
		$res =& $this->db->query($sql);
		if (DB::isError($res)) {
			$this->db->rollBack();
			throw new SException($res->getMessage());
		}
		$publication_id = last_insert_id($this->db);
		//insert into `author_has_publication`
		$sth = $this->db->prepare("INSERT INTO `author_has_publication` VALUES (?, ?)");
		if (PEAR::isError($sth)) {
			$this->db->rollBack();
			throw new SException($sth->getMessage());
		}
		if (is_array($authors)) foreach ($authors as $_id) {
			$res =& $this->db->execute($sth, array($_id, $publication_id));
			if (PEAR::isError($res)) {
				$this->db->rollBack();
				throw new SException($res->getMessage());
			}
		}
		//insert into `categories`
		$inserted_cat_ids = null;
		$tmp = $_SESSION['tmp_cat'];
		if (is_array($tmp)) {
			$sth =& $this->db->prepare("INSERT INTO `categories` VALUES('', '$submitter_id', ?, '0')");
			if (PEAR::isError($sth)) {
				$this->db->rollBack();
				throw new SException($sth->getMessage());
			}
			foreach ($tmp as $name) {
				$res =& $this->db->execute($sth, stripslashes($name));//stripslashes because prepare-execute add slashes redundantly
				$inserted_cat_ids[] = last_insert_id($this->db);
				if (DB::isError($res)) {
					$this->db->rollBack();
					throw new SException($res->getMessage());
				}
			}
		}
		//insert into `categories_has_publication`
		$tmp = $_SESSION['tmp_cat'];
		$sth =& $this->db->prepare("INSERT INTO `categories_has_publication` VALUES(?, ?)");
		if (PEAR::isError($sth)) {
			$this->db->rollBack();
			throw new SException($sth->getMessage());
		}
		if (is_array($inserted_cat_ids)) {
			foreach ($inserted_cat_ids as $category_id) {
				$res =& $this->db->execute($sth, array($publication_id, $category_id));
				if (DB::isError($res)) {
					$this->db->rollBack();
					throw new SException($res->getMessage());
				}
			}
		}
		$propagated_categories = $_SESSION['feedback_st1']['propagated_categories'];
		if (is_array($propagated_categories)) {
			foreach ($propagated_categories as $category_id) {
				$res =& $this->db->execute($sth, array($publication_id, $category_id));
				if (DB::isError($res)) {
					$this->db->rollBack();
					throw new SException($res->getMessage());
				}
			}
		}
		//insert public attributes
		$public_attr = $this->filled_ext_attr();
		if (is_array($public_attr)) { //prepare attribute insertion
			$sth = $this->db->prepare("INSERT INTO `attrib_storage` VALUES ('$publication_id', ?, '$submitter_id', ?)");
			if (PEAR::isError($sth)) {
				$this->db->rollBack();
				throw new SException($sth->getMessage());
			}
			foreach ($public_attr as $id => $name) {
				//insert into `attrib_storage`
				$res1 =& $this->db->execute($sth, array($id, stripslashes($_SESSION['feedback_st2'][$name])));//stripslashes because prepare-execute add slashes redundantly
				if (DB::isError($res1)) {
					$this->db->rollBack();
					throw new SException($res1->getMessage());
				}
			}
		}
		//----
		$attributes = $_SESSION['tmp_attr']['name'];
		if(is_array($attributes)) foreach ($attributes as $name => $value) {
			$description = $_SESSION['tmp_attr']['description'][$name]; //attribute description
			//insert into `attributes
			$sql = "INSERT INTO `attributes` VALUES ('', '$submitter_id', '$name', '$description', '0')";
			$res =& $this->db->query($sql);
			if (DB::isError($res)) {
				$this->db->rollBack();
				throw new SException($res->getMessage());
			}
			$attributes_id = last_insert_id($this->db);
			//insert into `attrib_storage`
			$sql = "INSERT INTO `attrib_storage` VALUES ('$publication_id', '$attributes_id', '$submitter_id', '$value')";
			$res =& $this->db->query($sql);
			if (DB::isError($res)) {
				$this->db->rollBack();
				throw new SException($res->getMessage());
			}
		}
		//store pdf/ppt into the filesystem
		$this->save_documents($publication_id);
		$this->db->commit();
		session_destroy();
		$this->smarty->assign('logged_info', $this->auth_user);
		$this->smarty->display('load_stFinish.tpl');
	}

	/**
	 * Store given documents
	 *
	 * @param int  $publication_id
	 */
	private function save_documents($publication_id) {
		if(!file_exists("../storage/$publication_id"))
			mkdir("../storage/$publication_id");
		foreach($_FILES['documents']['error'] as $key => $error) {
			if ($error == UPLOAD_ERR_OK) {
				$file =& $_FILES['documents']['tmp_name'][$key];
				$name =& $_FILES['documents']['name'][$key];
				$extension = substr($name, strrpos($name, ".") + 1);
				$res = move_uploaded_file($file, "../storage/$publication_id/".$name);
				if (!$res) {
					throw new SException('can not move uploaded file');
				}
			}
		}
	}
}
?>