<?php

require_once('./common/common.inc.php');
require_once('./common/db_fns.inc.php');
require_once('./libs/Smarty.class.php');
require_once('./common/Parser.inc.php');
require_once('./common/SException.inc.php');


class User {
	private $get;
	private $post;
	private $smarty;
	private $db;
	private $auth_user;

	public function __construct($post, $get, $login) {
		$this->auth_user = $login;
		$this->get = $this->clean($get);
		$this->post = $post;
		$this->smarty = new Smarty;
		//$this->smarty->compile_check = true;
		//$this->smarty->debugging = true;
		$this->db = db_connect();
	}

	/**
	 * Execute page
	 *
	 */
	public function execute() {
		if(!$this->post['format'] && !$_SESSION['selected_format']) {
			$_SESSION['selected_format'] = 1;
		} elseif ($this->post['format']) {
			$_SESSION['selected_format'] = intval($this->post['format']);
		}
		$this->display();
	}

	/**
	 * celean up messy input
	 *
	 * @param array $get  data sent via GET method
	 */
	private function clean($get)
	{
		$get['author'] = (int)$get['author'];
		$get['bulk'] = (int)$get['bulk'];
		if($get['bulk'] < 1 || $get['bulk'] > 100)
			$get['bulk'] = 10;
		$get['page'] = (int)$get['page'];
		if (is_array($get['categories'])) foreach ($get['categories'] as $i => $v) {
			$get[$i] = (int)$v;
		}
		return $get;
	}

	/**
	 * Initialize page bars e.g. right panel, navigation bar
	 *
	 * @param int $res  number of rows of the whole select
	 */
	private function init_bars($num_rows) {
		//initialize search bar
		if($this->get['categories'])
			$this->smarty->assign('enable_categories', 1);
		if($this->get['author'])
			$this->smarty->assign('enable_author', 1);
		$sql = "SELECT id, CONCAT_WS(' ',name,middlename,surname) AS author
				FROM `author`
                WHERE confirmed=1
				ORDER BY surname";
		$this->smarty->assign('author', $this->db->getAssoc($sql));
		$sql = "SELECT id, name
				FROM `categories`
				WHERE confirmed='1'
				ORDER BY name";
		$categories = $this->db->getAssoc($sql);
		$this->smarty->assign('categories', $categories);
		$this->smarty->assign('keywords', $this->get['keywords']);
		$this->smarty->assign('selected_categories', $this->get['categories']);
		$this->smarty->assign('selected_author', $this->get['author']);
		$this->smarty->assign('bulk', array(1=>1, 5=>5, 10=>10, 20=>20, 50=>50, 100=>100));
		if(!$this->get['bulk'])
			$this->smarty->assign('selected_bulk', 1);
		else
			$this->smarty->assign('selected_bulk', $this->get['bulk']);
		//initialize page navigation bar
		if ($this->get['bulk'] < 1 || !is_numeric($this->get['bulk']))
			$this->get['bulk'] = 10;
		$num_pages = ceil($num_rows / $this->get['bulk']);
		if($num_pages > 1) {
			$query_string = $_SERVER['QUERY_STRING'];
			if (!ereg('page=[0-9]*', $query_string)) {
				$query_string_left = htmlentities($query_string.'&page='.($this->get['page']-1));
				$query_string_right = htmlentities($query_string.'&page='.($this->get['page']+1));
			} else {
				$query_string_left = htmlentities(preg_replace('/page=[0-9]*/', 'page='.($this->get['page']-1), $query_string));
				$query_string_right = htmlentities(preg_replace('/page=[0-9]*/', 'page='.($this->get['page']+1), $query_string));
			}
			if ($this->get['page']==0) {
				$this->smarty->assign('rarrow', $query_string_right);
			}
			elseif ($this->get['page']!=0 && $this->get['page'] < $num_pages-1) {
				$this->smarty->assign('larrow', $query_string_left);
				$this->smarty->assign('rarrow', $query_string_right);
			}
			else
				$this->smarty->assign('larrow', $query_string_left);
		}
		$format = $this->db->getAssoc('SELECT id, name FROM `format`');
		$this->smarty->assign('format', $format);
		$this->smarty->assign('selected_format', $_SESSION['selected_format']);
	}

	/**
	 * Returns those rows form table `publication` which are
	 * in detail specified by extended parameters
	 *
	 * @param string $keywords
	 * @param int $author_id
	 * @param array $categories  numbers of categories
	 * @param int $bulk  amount of publications to show
	 * @param int $page  page to show
	 * @return array(DB_result, int) returned rows and count of rows without limit
	 */
	private function filter($keywords, $author_id, $categories, $bulk, $page) {
		$sql = "SELECT *, YEAR(issue_date) as year,
				DATE_FORMAT(issue_date, \"%c. %Y\") as month_year
				FROM `publication` WHERE confirmed=1";
		if(!$keywords && !$author_id && !$categories && !$bulk && !$page) {//search form has not been submited
			$res =& $this->db->query($sql);
		}
		else {
			if($keywords) {
				$conditions='';
				if ($keywords) //serch by title
					$conditions .= " AND title COLLATE utf8_czech_ci LIKE '%$keywords%'";
			}
			if(is_array($categories) && !$author_id) {
				$ids='';
				$count=count($categories)-1;
				foreach($categories as $v) {
					$ids .= $v.($count--?', ':''); //insert commas only between numbers
				}
				$sql = "SELECT *, YEAR(issue_date) as year, DATE_FORMAT(issue_date, \"%c. %Y\") as month_year
						FROM `publication` JOIN `categories_has_publication`
						ON publication.id = categories_has_publication.publication_id
						WHERE confirmed=1 AND categories_has_publication.categories_id
						IN ($ids)";
			}
			if(!is_array($categories) && $author_id) {
				$sql = "SELECT *, YEAR(issue_date) as year, DATE_FORMAT(issue_date, \"%c. %Y\") as month_year
						FROM `publication` JOIN `author_has_publication`
						ON publication.id = author_has_publication.publication_id
						WHERE confirmed=1 AND author_has_publication.author_id='$author_id'";
			}
			if(is_array($categories) && $author_id) {
				$ids='';
				$count=count($categories)-1;
				foreach($categories as $v) {
					$ids .= $v.($count--?', ':''); //insert commas only between numbers
				}
				$sql = "SELECT *, YEAR(issue_date) as year, DATE_FORMAT(issue_date, \"%c. %Y\") as month_year
						FROM `publication` JOIN `author_has_publication` JOIN `categories_has_publication`
						WHERE publication.id = author_has_publication.publication_id
						AND author_has_publication.author_id='$author_id'
						AND publication.id = categories_has_publication.publication_id
						AND confirmed=1 AND categories_has_publication.categories_id
						IN ($ids)";
			}
		}
		$offset = $bulk * $page;
		$limit = " LIMIT $offset, ".$bulk;
		$res =& $this->db->query($sql.$conditions.$limit);
		$res_unf =& $this->db->query($sql.$conditions);
		if (PEAR::isError($res_unf)) {
			throw new SException($res_unf->getMessage());
		}
		return array($res, $res_unf->numRows());
	}

	/**
	 * Display the page
	 *
	 */
	private function display() {
		$parser = new Parser();
		$res_arr =& $this->filter($this->get['keywords'], $this->get['author'], $this->get['categories'], $this->get['bulk'], $this->get['page']); //filtered publications
		$res = $res_arr[0];
		$this->init_bars($res_arr[1]);  //amount of rows without limit
		$sth = $this->db->prepare("SELECT name, middlename, surname FROM `author` JOIN `author_has_publication` ON
									(author.id = author_has_publication.author_id)
									WHERE confirmed=1 AND author_has_publication.publication_id = ?");
		$sth2 = $this->db->prepare("SELECT name
									FROM `categories` JOIN `categories_has_publication`
									ON categories.id=categories_has_publication.categories_id
									WHERE confirmed=1 AND publication_id=?");
		if (PEAR::isError($res)) {
    		die($res->getMessage());
		}
		if (PEAR::isError($sth)) {
    		die($sth->getMessage());
		}
		while ($row =& $res->fetchRow(DB_FETCHMODE_ASSOC)) {
			$authors_res =& $this->db->execute($sth, $row['id']);
			if (PEAR::isError($authors_res)) {
				die($authors_res->getMessage());
			}
			$num_authors = $authors_res->numRows();
			$count = 1; $authors = '';
			while ($row2 =& $authors_res->fetchRow(DB_FETCHMODE_ASSOC)) {
				$authors .= $row2['name'].' '.($row2['middlename']?$row2['middlename'].' ':'').$row2['surname'].($count++ < $num_authors ? ', ':'');
			}
			$categories_res =& $this->db->execute($sth2, $row['id']);
			if (PEAR::isError($categories_res)) {
				die($categories_res->getMessage());
			}
			$num_categories = $categories_res->numRows();
			$count = 1; $categories = '';
			while ($row_cat =& $categories_res->fetchRow(DB_FETCHMODE_ASSOC)) {
				$categories .= $row_cat['name'].($count++ < $num_categories ? ', ':'');
			}
			$pub_id = $row['publisher_id']; //publisher id
			$id = $row['id']; //publication id
			$publisher =& $this->db->getOne("SELECT name FROM `publisher` WHERE id = '$pub_id' AND confirmed = 1");
			$row = array_merge($row, array('publisher' => $publisher));

			$sql = "SELECT DISTINCT name, value
					FROM attributes JOIN attrib_storage
					ON attrib_storage.attributes_id=attributes.id
					AND attributes.id=attrib_storage.attributes_id
					AND attrib_storage.attributes_id=attrib_storage.attributes_id
					WHERE attrib_storage.publication_id='$id'
					AND attrib_storage.publication_id='$id'
					AND confirmed=1";
			$attributes = $this->db->getAssoc($sql);
			$publications[] = array($parser->get_parsed($_SESSION['selected_format'], $row, $authors, $attributes, $this->file_list($id)), $categories);
		}
		$this->smarty->assign('publications', $publications);
		$this->smarty->assign('logged_info', $this->auth_user);
		$this->smarty->display('user.tpl');
	}

	/**
	 * Return the file list attached to a publication
	 *
	 * @param int  $publication_id
	 * @return array  array of string with file names
	 */
	private function file_list($publication_id) {
		$path="./storage/$publication_id";
		if(file_exists($path)) {
			$handle=opendir($path);
			while ($file = readdir($handle)) {
				if (is_file($path.'/'.$file)){
					$tab[]=$file;
				}
			}
			return $tab;
		}
	}

}


?>