<?php
require_once('Admin.inc.php');

final class AdminT12 extends Admin
{
	public $tab;

	/**
	 * Constructor
	 *
	 * @param string $login
	 * @param string $pass
	 * @param int $tab  show tab==1 or tab==2, (choice between unconfirmed and confirmed publications list)
	 */
	public function __construct($post, $get, $tab, $require_admin_rights)
	{
		$this->tab = $tab;
		if ($tab == 1 || $tab == 2)
			parent::__construct($post, $get, $require_admin_rights);
		else
		{
			$this->tab = 1;
			parent::__construct($post, $get, $require_admin_rights);
		}
	}

	/**
	 * Execute admin tab1 or tab2
	 *
	 */
	public function execute()
	{
		if ($this->post['delete_files'] && $this->get['pid']){
			try {
				$this->delete_files($this->get['pid']);
				$this->smarty->assign('operation_success', 'Files has been deleted.');
				$this->display_overview($this->get['pid']);
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
				$this->display_overview($this->get['pid']);
			}
		} elseif ($this->post['add_file'] && $this->get['pid']) {
			try {
				$this->add_file($this->get['pid']);
				$this->smarty->assign('operation_success', 'File has been added.');
				$this->display_overview($this->get['pid']);
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
				$this->display_overview($this->get['pid']);
			}
		} elseif ($this->post['confirm'] && $this->get['pid']){
			try {
				$this->confirm($this->get['pid'], $this->post['propagated_attributes']);
				$this->smarty->assign('operation_success', 'Confirmed.');
				$this->smarty->assign('refresh', "1 ; URL=index.php");
				$this->smarty->display('admin_tab12.tpl');
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				die($e->getMessage());
			}
		} elseif ($this->post['unconfirm'] && $this->get['pid']){
			try {
				$this->confirm($this->get['pid'], $this->post['propagated_attributes'], 0); //unconfirm
				$this->smarty->assign('operation_success', 'Publicatioh has been moved to unchecked tab.');
				$this->smarty->assign('refresh', "1 ; URL=index.php?tab=2");
				$this->smarty->display('admin_tab12.tpl');
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				die($e->getMessage());
			}
		} elseif ($this->post['deny'] && $this->get['pid']){
			try {
				$this->delete($this->get['pid']);
				$this->smarty->assign('operation_success', 'Publication has been deleted.');
				$this->smarty->assign('refresh', "1 ; URL=index.php?tab=".$this->tab);
				$this->smarty->display('admin_tab12.tpl');
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				die($e->getMessage());
			}
		} elseif ($this->post['confirm_all_publications']){
			try {
				$this->confirm_all();
				$this->smarty->assign('operation_success', 'All publications have been checked.');
				$this->smarty->assign('refresh', "1 ; URL=index.php?tab=2");
				$this->smarty->display('admin_tab12.tpl');
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				die($e->getMessage());
			}
		} elseif ($this->post['modify'] && $this->get['pid']){
			try {
				$this->display_modify_publication($this->get['pid']);
			} catch(SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				die($e->getMessage());
			}
		} elseif ($this->post['delete_author']){
			try {
			 	$a = $this->post['authors'];
				if (sizeof($a) != 0)
				{
					$affected = 0;
					foreach ($a as $i => $value)
						if (!$affected) $affected = delete_author($this->db, $value);
						else delete_author($this->db, $value);
				}

				if ($affected)
					$this->smarty->assign("operation_success", "Authors have been deleted.");
				else if (sizeof($a) == 0)
					$this->smarty->assign("operation_error", "No author has beed choosen to be deleted.");
				else
					$this->smarty->assign("operation_error", "The record can't be deleted because is used by some publications.");
				$this->display_modify_publication($this->get['pid']);
			} catch (Exception $e) {
				$this->smarty->assign("operation_error", $e->getMessage());
				$this->display_modify_publication($this->get['pid']);
			}
		} else if ($this->post['add_author']){
			try {
				add_author($this->db, $this->post['author_name'], $this->post['author_middle_name'], $this->post['author_surname'], $this->auth_module->get_id());
				$this->smarty->assign("operation_success", "Author has been added.");
				$this->display_modify_publication($this->get['pid']);
			}
			catch (SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign("operation_error", $e->getMessage());
				$this->display_modify_publication($this->get['pid']);
			}
		} elseif ($this->post['add_publisher']) {
			try {
				add_publisher($this->db, $this->post['publisher_name'], $this->post['publisher_address'], $this->auth_module->get_id());
				$this->smarty->assign('operation_success', 'Publisher has been added.');
				$this->display_modify_publication($this->get['pid']);
			} catch (SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign("operation_error", $e->getMessage());
				$this->display_modify_publication($this->get['pid']);
			}
		} elseif ($this->post['add_journal']) {
			try {
				add_journal($this->db, $this->post['journal_name'], $this->post['journal_issn'], $this->auth_module->get_id());
				$this->smarty->assign('operation_success', 'Journal has been added.');
				$this->display_modify_publication($this->get['pid']);
			} catch (SLException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign("operation_error", $e->getMessage());
				$this->display_modify_publication($this->get['pid']);
			}
		} elseif ($this->post['add_conference']) {
			try {
				extract($this->post);
				add_conference($this->db, $conference_name, $conference_abbreviation, $w_year, $w_from, $w_to, $location, $isbn, $this->auth_module->get_id());
				$this->smarty->assign('operation_success', 'Conference has been added.');
				unset ($_SESSION['feedback_modify']['conference_name']);
				unset ($_SESSION['feedback_modify']['conference_abbreviation']);
				unset ($_SESSION['feedback_modify']['w_year']);
				unset ($_SESSION['feedback_modify']['w_from']);
				unset ($_SESSION['feedback_modify']['w_to']);
				unset ($_SESSION['feedback_modify']['isbn']);
				$this->display_modify_publication($this->get['pid']);
			} catch (SLException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign("operation_error", $e->getMessage());
				$this->display_modify_publication($this->get['pid']);
			}
		} elseif ($this->post['update_publisher']) {
			try {
				update_publisher($this->db, $this->post['publisher'], $this->post['publisher_name'], $this->post['publisher_address'], $this->auth_module->get_id());
				$this->smarty->assign('operation_success', 'Publisher has been updated.');
				$this->display_modify_publication($this->get['pid']);
			} catch (SAException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign("operation_error", $e->getMessage());
				$this->display_modify_publication($this->get['pid']);
			}
		} elseif ($this->post['update_journal']) {
			try {
				update_journal($this->db, $this->post['journal'], $this->post['journal_name'], $this->post['journal_issn'], $this->auth_module->get_id());
				$this->smarty->assign('operation_success', 'Journal has been updated');
				$this->display_modify_publication($this->get['pid']);
			} catch (SLException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign("operation_error", $e->getMessage());
				$this->display_modify_publication($this->get['pid']);
			}
		} elseif ($this->post['update_conference']) {
			try {
				extract($this->post);
				update_conference($this->db, $conference, $conference_name, $conference_abbreviation, $w_year, $w_from, $w_to, $location, $isbn, $this->auth_module->get_id());
				$this->smarty->assign('operation_success', 'Conference has been updated');
				$this->display_modify_publication($this->get['pid']);
			} catch (SLException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign("operation_error", $e->getMessage());
				$this->display_modify_publication($this->get['pid']);
			}
		} elseif ($this->post['delete_publisher']) {
			try {
				$affected = delete_publisher($this->db, $this->post['publisher']);
				if ($affected)
					$this->smarty->assign("operation_success", "Publisher has been deleted.");
				else {
					$this->smarty->assign("operation_error", "The record can't be deleted because is used by some publications.");
				}
				$this->display_modify_publication($this->get['pid']);
			} catch (Exception $e) {
				$this->smarty->assign("operation_error", $e->getMessage());
				$this->display_modify_publication($this->get['pid']);
			}
		} elseif ($this->post['delete_journal']) {
			try {
				$affected = delete_journal($this->db, $this->post['journal']);
				if ($affected)
					$this->smarty->assign("operation_success", "Journal has been deleted");
				else {
					$this->smarty->assign("operation_error", "The record can't be deleted because is used by some publications.");
				}
				$this->display_modify_publication($this->get['pid']);
			} catch (Exception $e) {
				$this->smarty->assign("operation_error", $e->getMessage());
				$this->display_modify_publication($this->get['pid']);
			}
		} elseif ($this->post['delete_conference']) {
			try {
				$affected = delete_conference($this->db, $this->post['conference']);
				if ($affected)
					$this->smarty->assign("operation_success", "Conference has been deleted.");
				else {
					$this->smarty->assign("operation_error", "The record can't be deleted because is used by some publications.");
				}
				$this->display_modify_publication($this->get['pid']);
			} catch (Exception $e) {
				$this->smarty->assign("operation_error", $e->getMessage());
				$this->display_modify_publication($this->get['pid']);
			}
		} elseif ($this->post['modify_form'] && $this->get['pid']) {
			try {
				if ($this->post['save']) { //save changes
					$this->validate_modify_publication();
					$this->update_publication($this->get['pid']);
					$this->clear_temporaries();
					$this->smarty->assign('operation_success', 'Publication has been updated.');
					$this->smarty->assign('refresh', "1 ; URL=index.php?tab=".$this->get['tab']."&amp;pid=".$this->get['pid']);
					$this->smarty->display('admin_tab12.tpl');
				} elseif ($this->post['add_attribute']) { //add temporary attribute
					$this->validate_add_att();
					$this->add_tmp_attribute($this->post['new_attribute_name'], $this->post['new_attribute_description'], $this->post['new_attribute_value']);
					$this->smarty->assign('operation_success', 'Attribute is going to be added after you save the publication.');
					$this->display_modify_publication($this->get['pid']);
				} elseif ($this->post['mark_attributes_deleted']) {  //unset all temporary attributes, mark real attributes as deleted
					$_SESSION['feedback_modify'] = $this->post;
					$_SESSION['attributes_deleted'] = 1;
					unset($_SESSION['tmp_attr']);
					$this->smarty->assign('operation_success', 'Attributes are going to be deleted after you save the publication.');
					$this->display_modify_publication($this->get['pid']);
				} elseif ($this->post['select_author']) { //add author from publication
					$authors = $this->post["authors"];
					if (!is_null($authors))
						$this->add_authors_to_temp($authors);
					$this->display_modify_publication($this->get['pid']);
				} elseif ($this->post['deselect_author']) { //remove author from publication
					$authors = $this->post["selected_authors"];
					if (!is_null($authors))
						$this->remove_authors_from_temp($authors);
					$this->display_modify_publication($this->get['pid']);
				}
			} catch(SAException $s) {
				die($s->getMessage().'<br/>Line:'.$s->getLine());
			} catch (Exception $e) {
				$this->feedback_modify();
				$this->smarty->assign('operation_error', $e->getMessage());
				$this->display_modify_publication($this->get['pid']);
			}
		}
		elseif ($this->get['pid'])
		{
			$this->clear_temporaries();
			$this->display_overview($this->get['pid']);
		}
		else
		{
			$this->clear_temporaries();
			$this->display_publ_list();
		}
	}


	/**
	 * Adds authors to the temporary list of authors in the session
	 *
	 * @param int[] $authors array of ids
	 */
	public function add_authors_to_temp($authors)
	{
		foreach ($authors as $id)
		{
			$contains = false;
			foreach ($_SESSION['selected_authors'] as $_id)
			{
				if ($id == $_id)
				{
					$contains = true;
					break;
				}
			}
			if (!$contains)
			{
				$_SESSION['selected_authors'] = array_merge($_SESSION['selected_authors'], array($id));
			}
		}
		return true;
	}


	/**
	 * Removes given authors from the temporary list of authors in the session
	 *
	 * @param int[] $authors array of ids to be deleted
	 */
	public function remove_authors_from_temp($authors)
	{
		$_SESSION['selected_authors'] = array_diff($_SESSION['selected_authors'], $authors);
	}


	/**
	 * Saves author ids to session
	 *
	 * @param int[] $author_ids array of ids
	 */
	public function manage_temporary_authors_list($author_ids)
	{
		if (is_null($_SESSION['selected_authors']))
		{
			$_SESSION['selected_authors'] = $author_ids;
		}
	}


	/**
	 * Display modify publication form
	 *
	 * @param int $pid  publication_id
	 */
	public function display_modify_publication($pid)
	{
		$this->smarty->assign('modify', 1); //decide short_look/modify part of the template
		$this->smarty->assign('tab', $this->get['tab']);
		$this->fill_modify_form($pid);
		$this->smarty->display('admin_tab12.tpl');
	}

	/**
	 * Fill form dedicated to modify publication. Assignes necessary
	 * variables to template modify_publication.tpl
	 *
	 */
	public function fill_modify_form($pid)
	{
		$submitter_id = $this->db->getOne("SELECT submitter_id FROM `publication` WHERE id='$pid'");
		//list of authors
		$sql = "SELECT id, CONCAT_WS(' ',name,middlename,surname) AS author
				FROM `author`
				ORDER BY surname";
		$this->smarty->assign('authors', htmlspecialchars_rec($this->db->getAssoc($sql)));
		//list of publishers
		$sql = "SELECT id, name
				FROM `publisher`
				ORDER BY name";
		$this->smarty->assign('publisher', htmlspecialchars_rec($this->db->getAssoc($sql)));
		//list of journals
		$sql = "SELECT id, name
				FROM `journal`
				ORDER BY name";
		$arr = $this->db->getAssoc($sql);
		if (PEAR::isError($arr))
			throw new SAException($arr->getMessage());
		$this->smarty->assign('journal', htmlspecialchars_rec($arr));

		//list of all conferences including non-complete conferences (with name but without year)
		$sql = "SELECT id, CONCAT(name, ' (', w_year, ')')
				FROM `conference`
				ORDER BY name";
		$arr = $this->db->getAssoc($sql);
		if (PEAR::isError($arr))
			throw new SAException($arr->getMessage());
		$this->smarty->assign('conference', htmlspecialchars_rec($arr));

		//forbidden conferences, pass them as a JS array to the output
		$arr = $this->get_forbidden_conferences();
		$counter = 0;
		$jsarr = 'Array(';
		if (is_array($arr)) foreach ($arr as $id)
		{
			$jsarr .= '"'.$id.'"';
			if (++$counter < count($arr)) $jsarr .= ', ';
		}
		$jsarr .= ')';
		$this->smarty->assign('forbidden_conferences', $jsarr);

		//list of categories
		$sql = "SELECT id, name
				FROM `categories`
				ORDER BY name";
		$this->smarty->assign('categories', htmlspecialchars_rec($this->db->getAssoc($sql)));
		$this->smarty->assign('pid', $pid);
		$this->smarty->assign("pub_types", pub_types_full());
		//authors
		$author_ids =& $this->db->getAssoc("SELECT DISTINCT id, id
		FROM `author` JOIN `author_has_publication` ON author.id=author_has_publication.author_id
		WHERE publication_id='$pid'");
		if (PEAR::isError($author_ids))
			throw new SAException($author_ids->getMessage());
		$this->manage_temporary_authors_list($author_ids);
		if ($_SESSION["selected_authors"] != null)
		{
			$author_names = array();
			foreach ($_SESSION["selected_authors"] as $id)
			{
				$author_name = $this->db->getOne("select CONCAT_WS(' ',name,middlename,surname) AS author from author where id=?", $id);
				if (PEAR::isError($author_name))
					throw new Exception($author_name->getMessage(), 1);
				$author_names = array_merge($author_names, array($author_name));
			}
		}
		$this->smarty->assign('selected_author_ids', $_SESSION["selected_authors"]);
		$this->smarty->assign('selected_author_names', $author_names);

		if (!$_SESSION['feedback_modify']) //fill the form with data if no changes were already made
		{
			//get from db and display
			$res =& $this->db->query("SELECT * FROM `publication` WHERE id='$pid'");
			if(PEAR::isError($res))
				throw new SAException($res->getMessage());
			if($res->numRows() == 0)
				throw new SAException('Publikace neexistuje.');
			$row =& $res->fetchRow(DB_FETCHMODE_ASSOC);
			$row = htmlspecialchars_rec($row);
			$this->smarty->assign('decision_pub_type', $row['pub_type']);
			$this->smarty->assign('selected_pub_type', $row['pub_type']);
			foreach ($row as $i => $v)
			{
				if ($i != 'pub_type' && $i != 'authors' && $i != 'journal'
					&& $i != 'conference' && $i != 'publisher')
				{
					if ($i == 'issue_date')
					{
						if ($v == '0000-00-00')
							$v = '';
						else
							$v = substr($v, 0, 7);
					}
					$this->smarty->assign($i, $v);
				}
			}
			$this->smarty->assign('selected_publisher', $row['publisher_id']);
			$this->smarty->assign('selected_journal', $row['journal_id']);
			$this->smarty->assign('selected_conference', $row['conference_id']);
			//categories
			$sql = "SELECT categories_id
					FROM `categories_has_publication`
					WHERE publication_id = '$pid'";
			$selected_categories =& $this->db->getAll($sql, null, DB_FERCHMODE_ORDERED);
			if (PEAR::isError($selected_categories))
				throw new SAException($selected_categories->getMessage());
			foreach ($selected_categories as $i => $v) {
				$selected_categories[$i] = $v[0];
			}
			$this->smarty->assign('selected_categories', $selected_categories);
		} else
			$this->feedback_modify();

		//--atttibutes--
		//public attributes
		$sql = "SELECT id, name FROM attributes WHERE confirmed = '1'";
		$arr =& $this->db->getAssoc($sql);
		$sql = "SELECT id, description FROM attributes WHERE confirmed = '1'";
		$desc =& $this->db->getAssoc($sql);
		$s='';
		foreach ($arr as $i => $v) {
			$value = $this->db->getOne("SELECT value FROM `attrib_storage` WHERE attributes_id='".$i."' AND publication_id='$pid'");
			$s .= '<tr><td><strong>'.$v.'</strong>'.($desc[$i]?'<br />('.htmlspecialchars($desc[$i]).')':'').'</td><td colspan="2"><input type="text" name="'.$v.'" size="80" value="'.htmlspecialchars($value).'" /></td></tr>';
		}
		//unpublished attributes
		$sql = "SELECT DISTINCT id, name, description, confirmed
				FROM `attrib_storage` JOIN `attributes`
				ON attrib_storage.attributes_id=attributes.id
				WHERE attrib_storage.publication_id = '$pid' and confirmed=0";
		$all = $this->db->getAll($sql, array(), DB_FETCHMODE_ASSOC);
		$n='';
		if (is_array($all)) foreach ($all as $arr) {
			if (!$_SESSION['attributes_deleted'])
				$n .= '<tr><td colspan="3"><strong>'.$arr['name'].':</strong> <cite>'.
				htmlspecialchars($this->db->getOne("SELECT value FROM `attrib_storage` WHERE attributes_id='".$arr['id']."' AND publication_id='$pid'")).'</cite>'.
				'<br />('.htmlspecialchars($arr['description']).')</td></tr>';
		}
		//temporarily added new attributes
		$t='';
		$tmp = $_SESSION['tmp_attr']['name']; //$tmp is an array
		$tmp = stripslashes_rec(htmlspecialchars_rec($tmp));
		if (is_array($tmp)) {
			foreach ($tmp as $i => $value) {
				$description = stripslashes(htmlspecialchars($_SESSION['tmp_attr']['description'][$i]));
				$t .= '<tr><td><strong>'.$i.'</strong>: <cite>'.$value.'</cite>'.($description?'<br />('.$description.')':'').'</td></tr>';
			}
		}
		$this->smarty->assign('tmp_attr', $n.$t);
		$this->smarty->assign('attributes', $s);
	}

	/**
	 * Return ids of all conferences which are non-complete and doesn't belong to the current user
	 * these conferences are going to be forbidden for the user.
	 *
	 * @return array of rows
	 */
	public function get_forbidden_conferences()
	{
		$sql = "SELECT id
				FROM `conference`
				WHERE w_year='0000' AND submitter_id <> '".$this->auth_module->get_id()."'";
		$res = $this->db->query($sql);
		if (PEAR::isError($res))
			throw new SAException($res->getMessage());
		while ($row = $res->fetchRow(DB_FETCHMODE_ORDERED))
			$arr[] = $row[0];
		if (is_array($arr))
			return $arr;
		else
			return array();
	}


	/**
	 * Validate form data sent through modify form
	 *
	 */
	public function validate_modify_publication()
	{
		$_SESSION['feedback_modify'] = $this->post; //save typed form data
		extract($this->post);

		if (!array_key_exists($pub_type, pub_types_full()))
			throw new Exception('Publication type is not selected.');
		if (!$title)
			throw new Exception('Title is missing.');
	}

	/**
	 * Validate temporarily added extensible attributes
	 *
	 */
	public function validate_add_att()
	{
		$_SESSION['feedback_modify'] = $this->post; //save typed form data
		extract($this->post);
		if (!ereg('^_([0-9]|[a-zA-Z]){0,31}$', $new_attribute_name))
			throw new Exception('First character must be "_", name may contain letters of alphabet and numbers, max. length is 32 chars.');
		if (!$new_attribute_value)
			throw new Exception('Value of attribute is missing.');
		if (sizeof($new_attribute_description) > 255)
			throw new Exception('Attribute description is too long.');
		unset($_SESSION['feedback_modify']['new_attribute_name']);
		unset($_SESSION['feedback_modify']['new_attribute_description']);
		unset($_SESSION['feedback_modify']['new_attribute_value']);
	}

	/**
	 * Gives written data back to form modify form
	 *
	 */
	public function feedback_modify()
	{
		$feedback = htmlspecialchars_rec($_SESSION['feedback_modify']);
		$this->smarty->assign('selected_pub_type', $feedback['pub_type']);
		$this->smarty->assign('selected_authors', $feedback['authors']);
		$this->smarty->assign('selected_publisher', $feedback['publisher']);
		$this->smarty->assign('selected_journal', $feedback['journal']);
		$this->smarty->assign('selected_conference', $feedback['conference']);
		$this->smarty->assign('selected_categories', $feedback['propagated_categories']);
		foreach ($feedback as $i => $v)
		{
			if ($i != 'pub_type' && $i != 'authors' && $i != 'journal' && $i != 'conference' && $i != 'publisher')
				$this->smarty->assign($i, $v);
		}
	}


	/**
	 * General purpose = when admin goes back from "modify puplication section"
	 * in order to discard his changes.
	 */
	public function clear_temporaries()
	{
		unset($_SESSION['feedback_modify']);
		unset($_SESSION['tmp_attr']);
		unset($_SESSION['attributes_deleted']);
	}

	/**
	 * Confirm publication
	 *
	 * @param int $publication_id
	 * @param array of int $propagated_attributes  list of ids of extra attributes,
	 * which are supposed to be public for all. Format must be: array(0 => attributes_id, 1 => arrtibutes_id, ...)
	 * @param  use 1 to confirm, 0 to unconfirm the publication
	 */
	public function confirm($publication_id, $propagated_attributes=null, $confirmed=1)
	{
		$this->db->autoCommit(false);
		//update `publication`
		$res =& $this->db->query("UPDATE `publication` SET confirmed='$confirmed' WHERE id='$publication_id'");
		if(DB::isError($res)) {
			$this->db->rollback();
			throw new SAException($res->getMessage());
		}
		//update `attributes` - take all unconfirmed attributes dependnt to the current
		//publication and if there are some, compare their ids to the $propagated_attributes
		if ($confirmed==1)
		{//do this only if confirming, backward process is impossible
			$sql = "SELECT DISTINCT id
					FROM `attrib_storage` JOIN `attributes`
					ON attrib_storage.attributes_id=attributes.id
					WHERE confirmed=0 AND attrib_storage.publication_id = '$publication_id'";
			$attributes = $this->db->getAll($sql, array(), DB_FETCHMODE_ORDERED);
			if (is_array($attributes)) foreach ($attributes as $id) {
				$id = $id[0];
				if (in_array($id, $propagated_attributes)) {//decide whether administrator wants this attribute to be public or not
					$res =& $this->db->query("UPDATE `attributes` SET confirmed=1 WHERE id='$id' AND confirmed=0");
					if (PEAR::isError($res)) {
						$this->db->rollback();
			    		throw new SAException($res->getMessage());
					}
				}
			}
		}
		$this->db->commit();
	}

	/**
	 * Confirm all publications, ignore all newly added attributes
	 *
	 */
	public function confirm_all()
	{
		$res =& $this->db->query("UPDATE `publication` SET confirmed='1' WHERE confirmed=0");
		if(DB::isError($res))
			throw new SAException($res->getMessage());
	}

	/**
	 * Delete publication
	 *
	 * @param int $publication_id
	 */
	public function delete($publication_id)
	{
		$this->db->autoCommit(false);
		/* Uncomment this code only if you are not using DELETE CASCADE refference definition
		//delete from `categories_has_publication`
		$res =& $this->db->query("DELETE FROM `categories_has_publication` WHERE publication_id='$publication_id'");
		if(PEAR::isError($res)) {
			$this->db->rollBack();
			throw new SAException($res->getMessage());
		}*/
		$sql = "SELECT publication_id, attributes_id
				FROM `attrib_storage`
				GROUP BY attributes_id
				HAVING COUNT(attributes_id) < 2 AND publication_id='$publication_id'";
		$relevant =& $this->db->getAll($sql, array(), DB_FETCHMODE_ASSOC);
		if(PEAR::isError($relevant)) {
			$this->db->rollBack();
			throw new SAException($relevant->getMessage());
		}
		/* Uncomment this code only if you are not using DELETE CASCADE refference definition
		//delete form `attrib_storage`
		$res =& $this->db->query("DELETE FROM `attrib_storage` WHERE publication_id='$publication_id'");
		if(PEAR::isError($res)) {
			$this->db->rollBack();
			throw new SAException($res->getMessage());
		}*/
		//delete from `attributes`
		if (is_array($relevant)) foreach($relevant as $row)
		{//delete relevant attributes (refferenced only once)
			$this->db->query("DELETE FROM `attributes` WHERE confirmed=0 AND id='".$row['attributes_id']."'");
		}
		/* Uncomment this code only if you are not using DELETE CASCADE refference definition in the database
		//delete form `author_has_publication`
		$res =& $this->db->query("DELETE FROM `author_has_publication` WHERE publication_id='$publication_id'");
		if(PEAR::isError($res)) {
			$this->db->rollBack();
			throw new SAException($res->getMessage());
		}*/

		//delete from `publication`
		$res =& $this->db->query("DELETE FROM `publication` WHERE id='$publication_id'");
		if(PEAR::isError($res)) {
			$this->db->rollBack();
			throw new SAException($res->getMessage());
		}
		//delete from documents, please note that this code is necessary. Table `documents`
		//is non-transactional table and therefore doesn't allow "delete cascade" refference definition
		$res =& $this->db->query("DELETE FROM `documents` WHERE publication_id='$publication_id'");
		if(PEAR::isError($res)) {
			$this->db->rollBack();
			throw new SAException($res->getMessage());
		}
		$this->delete_files($publication_id);
		$this->db->commit();
	}

	/**
	 * Creates a temporary information about new attributes
	 *
	 * @param string $name  name of the new added attribute
	 * @param string $description  description of the new added attribute
	 * @param string $value  text value of the attribute
	 */
	public function add_tmp_attribute($name, $description, $value)
	{
		$_SESSION['tmp_attr']['name'][$name] = $value;
		$_SESSION['tmp_attr']['description'][$name] = $description;
	}


	/**
	 * Display tab 1 or 2 entry point (show list of unconfirmed or list of confirmed publications)
	 *
	 */
	public function display_publ_list()
	{
		if ($this->tab == 1) //display unconfirmed publications
			$res =& $this->db->query("SELECT id, title, submitter_id
				FROM `publication`
				WHERE confirmed=0 OR confirmed=NULL
				ORDER BY title");
		elseif ($this->tab == 2) //display confirmed
			$res =& $this->db->query("SELECT id, title, submitter_id
				FROM `publication`
				WHERE confirmed=1
				ORDER BY title");
		$publications = '';
		while ($row =& $res->fetchRow(DB_FETCHMODE_ASSOC))
		{
			$publications .= '<a href="index.php?tab='.$this->tab.'&amp;pid='.$row['id'].'">'.htmlspecialchars($row['title']).'</a><br />';
		}
		$this->smarty->assign('tab', $this->tab);
		$this->smarty->assign('publications', $publications);
		$this->smarty->display('admin_tab12.tpl');
	}


	/**
	 * Display publications details list and allow/deny form
	 *
	 * @param int $pid  publication_id
	 */
	public function display_overview($pid)
	{
		$this->smarty->assign('pid', $pid);
		$this->smarty->assign('short_look', 1);  //decide short_look/modify part of the template

		$data = get_all_publication_data($this->db, $pid);
		$date_arr = explode('-', $data['issue_date']);
		foreach ($data as $i => $v)
		{
			$this->smarty->assign($i, $v);
		}
		$this->smarty->assign('execution_date', $data['from']?$w_from." -> ".$w_to:'');
		//categories
		$categories = get_categories($this->db, $pid);
		$this->smarty->assign('categories', $categories);
		//attributes
		$sql = "SELECT DISTINCT id, name, description, confirmed
				FROM `attrib_storage` JOIN `attributes`
				ON attrib_storage.attributes_id=attributes.id
				WHERE attrib_storage.publication_id = '$pid'";
		$attributes = $this->db->getAll($sql, array(), DB_FETCHMODE_ASSOC);
		if (PEAR::isError($attributes)) {
			throw new Exception($attributes->getMessage());
		}
		$s = '';
		if (is_array($attributes))
		foreach ($attributes as $arr)
		{
			$s .=
			'<tr>
				<td><input type="checkbox" name="propagated_attributes[]" value="'.$arr['id'].'" '.($arr['confirmed']?'checked="checked" disabled="disabled"':'').' /></td>
				<td><strong>'.$arr['name'].'</strong></td>'.'
				<td>'.htmlspecialchars($arr['description']).'</td>'.'
				<td>'.htmlspecialchars($this->db->getOne("SELECT value FROM `attrib_storage` WHERE attributes_id='".$arr['id']."' AND publication_id='$pid'")).'</td>
			</tr>';
		}
		$this->smarty->assign('attributes', $s);
		//files
		$dp = new DocumentProvider($this->db, true);
		$files = $dp->get_file_links($pid);
		$this->smarty->assign('attached_files', $files);
		//submitter
		$user_name = $this->db->getOne("SELECT name FROM `submitter` WHERE id=?", $data['submitter_id']);
		if (PEAR::isError($user_name))
			throw new SAException($user_name->getMessage());
		$this->smarty->assign('user_name', $user_name);

		$this->smarty->assign('tab', $this->tab);
		$this->smarty->display('admin_tab12.tpl');
	}


	/**
	 * Delete files formerly attached to the publication from its storage location
	 *
	 * @param int $pid
	 */
	public function delete_files($publication_id)
	{
		$dirpath = "../storage/$publication_id/";
		$handle = @opendir($dirpath);
		if (!$handle)
			return;
		while ($file = readdir($handle))
		{
			if (is_file($path="../storage/$publication_id/".$file))
				unlink($path);
		}
		closedir($handle);
		$res = $this->db->query("DELETE FROM `documents` WHERE publication_id=?", $publication_id);
		if (PEAR::isError($res))
			throw new SAException($res->getMessage());
    	if (!rmdir($dirpath))
    		throw new SAException("Directory couldn't be deleted.");
	}

	/**
	 * Add document to the publication
	 *
	 * @param int  $publication_id
	 * @param string $title publication title
	 */
	public function add_file($publication_id)
	{

		$file =& $_FILES['file']['tmp_name'];
		$name =& $_FILES['file']['name'];
		$extension = substr($name, strrpos($name, ".") + 1);
		if ((!is_uploaded_file($file) && $file))
			throw new SAException("Not an uploaded file!");
		if ($file) {
			$convertor = new Convertor($this->db, $publication_id);
			if(!file_exists("../storage/$publication_id"))
				mkdir("../storage/$publication_id");
			$res = move_uploaded_file($file, "../storage/$publication_id/".$name);
			$convertor->add_file($name);
			if (!$res) {
				throw new SAException("Can't move uploaded file.");
			}
			$convertor->finish();
		}
	}

	/**
	 * Update publication
	 *
	 * @param int $pid  publication id
	 */
	public function update_publication($pid)
	{
		$submitter_id = $this->auth_module->get_id();
		$authors = $_SESSION['selected_authors']; //array of authors
		$data = $_SESSION['feedback_modify'];  //pull out saved data (saved by validate_modify_publication())
		$publisher_id = $data['publisher'];
		$conference_id = $data['conference'];
		$journal_id = $data['journal'];
		extract($data);
		$issue_date = $data['issue_date'].'-01';

		if (in_array($conference_id, $this->get_forbidden_conferences()))
			throw new Exception("You can't choose this conference, because is non-complete and you are not it's owner.");

		$this->db->autoCommit(false);
		//update `publication`
		$sql = "UPDATE `publication` SET
				submitter_id='$submitter_id',
				publisher_id='$publisher_id',
				conference_id='$conference_id',
				journal_id='$journal_id',
				pub_type='$pub_type',
				title='$title',
				booktitle='$booktitle',
				abstract='$abstract',
				issue_date='$issue_date',
				isbn='$isbn',
				volume='$volume',
				number='$number',
				pages='$pages',
				note='$note',
				chapter='$chapter',
				edition='$edition',
				editor='$editor',
				howpublished='$howpublished',
				institution='$institution',
				organization='$organization',
				school='$school',
				address='$address',
				type_of_report='$type',
				url='$url'
				WHERE id='$pid'";
		$res =& $this->db->query($sql);
		if (DB::isError($res)) {
			$this->db->rollBack();
			throw new SAException($res->getMessage());
		}
		//delete old rows and insert new rows into `author_has_publication`
		$res =& $this->db->query("DELETE FROM `author_has_publication` WHERE publication_id='$pid'");
		if (DB::isError($res)) {
			$this->db->rollBack();
			throw new SAException($res->getMessage());
		}
		$sth = $this->db->prepare("INSERT INTO `author_has_publication` VALUES (?, ?, ?)");
		if (PEAR::isError($sth)) {
			$this->db->rollBack();
			throw new SAException($sth->getMessage());
		}
		$priority = 1;
		if (is_array($authors)) foreach ($authors as $_id) {
			$res =& $this->db->execute($sth, array($_id, $pid, $priority++));
			if (PEAR::isError($res)) {
				$this->db->rollBack();
				throw new SAException($res->getMessage());
			}
		}
		//insert/delete from `categories_has_publication`
		$propagated_categories = $this->post['propagated_categories'];//what has been checked in the form
		if(!$propagated_categories) $propagated_categories = array();
		$assigned_categories = $this->assigned_categories($pid);//what is in database
		if(!$assigned_categories) $assigned_categories = array();
		if (is_array($propagated_categories)) {
			$sth =& $this->db->prepare("INSERT INTO `categories_has_publication` VALUES(?, ?)");
			if (PEAR::isError($sth)) {
				$this->db->rollBack();
				throw new SAException($sth->getMessage());
			}
			foreach ($propagated_categories as $category_id) {
				if(!in_array($category_id, $assigned_categories)) {
					$res =& $this->db->execute($sth, array($pid, $category_id));
					if (DB::isError($res)) {
						$this->db->rollBack();
						throw new SAException($res->getMessage());
					}
				}

			}
		}
		$unwanted_categories = array_diff($assigned_categories, $propagated_categories);
		if (is_array($unwanted_categories)) {
			$sth =& $this->db->prepare("DELETE FROM `categories_has_publication` WHERE categories_id=? AND publication_id='$pid'");
			if (PEAR::isError($sth)) {
				$this->db->rollBack();
				throw new SAException($sth->getMessage());
			}
			$res =& $this->db->executeMultiple($sth, $unwanted_categories);//delete unwanted category bundles form table `categories_has_publication`
			if (DB::isError($res)) {
				$this->db->rollBack();
				throw new SAException($res->getMessage());
			}
		}
		//---- attributes -----
		if ($_SESSION['attributes_deleted']) {
			$sql = "SELECT publication_id, attributes_id
					FROM `attrib_storage`
					GROUP BY attributes_id
					HAVING COUNT(attributes_id) < 2 AND publication_id='$pid'";
			$relevant =& $this->db->getAll($sql, array(), DB_FETCHMODE_ASSOC);
			if(PEAR::isError($relevant)) {
				$this->db->rollBack();
				throw new SAException($relevant->getMessage());
			}
			//delete from `attrib_storage` - only public(unconfirmed) attributes
			$res =& $this->db->query("SELECT attributes_id as id FROM `attrib_storage`
									WHERE publication_id='$pid' AND attributes_id IN
									(SELECT attributes_id
									FROM `attrib_storage` JOIN `attributes`
									ON attrib_storage.attributes_id=attributes.id
									WHERE confirmed=0 AND publication_id='$pid')");
			if(PEAR::isError($res)) {
				$this->db->rollBack();
				throw new SAException($res->getMessage());
			}
			while ($row =& $res->fetchRow()) {
				$rd =& $this->db->query("DELETE FROM `attrib_storage` WHERE publication_id='$pid' AND attributes_id=".$row[0]);
				if(PEAR::isError($rd)) {
					$this->db->rollBack();
					throw new SAException($rd->getMessage());
				}
			}
			//delete from `attributes` if public and relevant (refferenced only once)
			if (is_array($relevant)) foreach($relevant as $row) {
				$res =& $this->db->query("DELETE FROM `attributes` WHERE confirmed=0 AND id='".$row['attributes_id']."'");
				if(PEAR::isError($res)) {
					$this->db->rollBack();
					throw new SAException($res->getMessage());
				}
			}
		}
		//insert new attributes added by admin
		$attributes = $_SESSION['tmp_attr']['name'];
		if(is_array($attributes)) foreach ($attributes as $name => $value) {
			$description = $_SESSION['tmp_attr']['description'][$name]; //attribute description
			//insert into `attributes
			$sql = "INSERT INTO `attributes` VALUES ('', '$submitter_id', '$name', '$description', '0')";
			$res =& $this->db->query($sql);
			if (DB::isError($res)) {
				$this->db->rollBack();
				throw new SAException($res->getMessage());
			}
			$attributes_id = last_insert_id($this->db);
			//insert into `attrib_storage`
			$sql = "INSERT INTO `attrib_storage` VALUES ('$pid', '$attributes_id', '$submitter_id', '$value')";
			$res =& $this->db->query($sql);
			if (DB::isError($res)) {
				$this->db->rollBack();
				throw new SAException($res->getMessage());
			}
		}
		$public_attr = $this->filled_ext_attr();
		if (is_array($public_attr)) { //prepare attribute insertion
			$sth1 = $this->db->prepare("INSERT INTO `attrib_storage` VALUES ('$pid', ?, '$submitter_id', ?)");
			$sth2 = $this->db->prepare("UPDATE `attrib_storage` SET value=?
										WHERE publication_id='$pid' AND attributes_id=?");
			if (PEAR::isError($sth1)) {
				$this->db->rollBack();
				throw new SAException($sth1->getMessage());
			}
			if (PEAR::isError($sth2)) {
				$this->db->rollBack();
				throw new SAException($sth2->getMessage());
			}
			foreach ($public_attr as $id => $name) {
				//insert into `attrib_storage`
				$nr =& $this->db->query("SELECT * FROM `attrib_storage` WHERE attributes_id='$id' AND publication_id='$pid'");
				$isset = $nr->numRows();
				if ($isset) {
					//UPDATE
					$res1 =& $this->db->execute($sth2, array($_SESSION['feedback_modify'][$name], $id));
				} else {
					//INSERT
					$res1 =& $this->db->execute($sth1, array($id, $_SESSION['feedback_modify'][$name]));
				}
				if (DB::isError($res1)) {
					$this->db->rollBack();
					throw new SAException($res1->getMessage());
				}
			}
		}
		//delete form `attrib_storage` - if there is an attributte in database but its field in form is empty, simply delete it
		$empty_public_attr = $this->empty_ext_attr();
		$res =& $this->db->query("SELECT `attributes_id` FROM `attrib_storage` WHERE publication_id='$pid'");
		if (DB::isError($res)) {
			$this->db->rollBack();
			throw new SAException($res->getMessage());
		}
		while ($row =& $res->fetchRow()) {
			$attributes_id = $row[0];
			if ($empty_public_attr[$attributes_id]) {
				$dr =& $this->db->query("DELETE FROM `attrib_storage` WHERE attributes_id='$attributes_id' AND publication_id='$pid'");
				if (PEAR::isError($dr)) {
					$this->db->rollBack();
					throw new SAException($dr->getMessage());
				}
			}
		}
		$this->db->commit();
	}


	/**
	 * Recognize which extensible attribute have been assigned
	 *
	 * @return array of $attribute_id => $attrinute_name
	 */
	public function filled_ext_attr()
	{
		foreach ($_SESSION['feedback_modify'] as $name => $value) {
			$sth = $this->db->prepare("SELECT id, name FROM `attributes` WHERE name=? AND confirmed='1'");
			if (PEAR::isError($sth)) {
				throw new SAException($sth->getMessage());
			}
			$res =& $this->db->execute($sth, $name);
			if ($res->numRows()) {
				$row = $res->fetchRow();
				$id = $row[0]; $nm = $row[1];
				if ($_SESSION['feedback_modify'][$nm] != '') //if value of the extensible attribute in form stage 2 has been assigned
					$public_attr[$id] = $nm;
			}
		}
		return $public_attr;
	}

	/**
	 * Recognize which extensible attribute have not been assigned
	 *
	 * @return array of $attribute_id => $attrinute_name
	 */
	public function empty_ext_attr()
	{
		foreach ($_SESSION['feedback_modify'] as $name => $value) {
			$sth = $this->db->prepare("SELECT id, name FROM `attributes` WHERE name=? AND confirmed='1'");
			if (PEAR::isError($sth)) {
				throw new SAException($sth->getMessage());
			}
			$res =& $this->db->execute($sth, $name);
			if ($res->numRows()) {
				$row = $res->fetchRow();
				$id = $row[0]; $nm = $row[1];
				if ($_SESSION['feedback_modify'][$nm] == '') //if value of the extensible attribute in form stage 2 hasn't been assigned
					$public_attr[$id] = $nm;
			}
		}
		return $public_attr;
	}

	/**
	 * Return ids of categories assigned to the specified publication
	 *
	 * @param int $publication_id
	 * @return array array of category ids
	 */
	public function assigned_categories($publication_id)
	{
		$sql = "SELECT categories_id
				FROM `categories_has_publication`
				WHERE publication_id = '$publication_id'";
		$data =& $this->db->getAll($sql, array(), DB_FETCHMODE_ORDERED);
		if(PEAR::isError($data)) throw new Exception($data->getMessage());
		foreach ($data as $i => $v) $data[$i] = $v[0];
		return $data;
	}
}
?>