<?php

require_once('../common/misc_fns.inc.php');
require_once('../common/SIException.inc.php');

/**
 * Lexical element
 *
 */
class LexElem
{
	public $symbol;
	public $sattribute;

	/**
	 * Constructor
	 *
	 * @param string $symbol
	 * @param string $sattribute
	 */
	public function __construct($symbol, $sattribute)
	{
		$this->symbol = $symbol;
		$this->sattribute = $sattribute;
	}
}

/**
 * Lexical analyzer
 *
 */
class Lexan
{
    public $text;
    public $symbol; //lexical symbol
    private $input_symbol; //imput symbol
    private $input_char;
    private $pos=0; // position in text
	public $line_number=1;

    /**
     * Constructor - prepares input text
     *
     * @param string $text BibTeX definition
     */
    public function Lexan($text)
    {
        $this->text = $text;
        $lp = strpos($this->text, '{');
        $rp = strrpos($this->text, '}');
        if (!$lp || !$rp)
        	throw new Exception('There are no {} brackets');
        $this->text[$lp] = '(';
        $this->text[$rp] = ')'; //replace with rounded brackets

        $this->read_input();
    }

    /**
     * Read input
     *
     */
    private function read_input()
    {
		if ($this->pos < strlen($this->text))
			$this->input_char = $this->text[$this->pos++];
		else
			$this->input_char = null;

		if ($this->input_char>='a' && $this->input_char<='z' || $this->input_char>='A' && $this->input_char<='Z')
			$this->input_symbol = 'c'; //char
		elseif ($this->input_char>='0' && $this->input_char<='9')
			$this->input_symbol = 'n'; //number
		elseif ($this->input_char == null)
			$this->input_symbol = 'e'; //end of file
		elseif ($this->input_char == "\n")
			$this->line_number++; //increase line counter
		elseif ($this->input_char <= ' ')
			$this->input_symbol = 's'; //white chars
		else
			$this->input_symbol = $this->input_char;
    }


    /**
     * Read symbol
     *
     * @return lexical element
     */
    public function read_symbol()
    {
		while ($this->input_symbol=='s')
			$this->read_input();
		switch ($this->input_symbol)
		{
			case 'e':
				$this->symbol = 'EOF';
				return new LexElem($this->symbol, null);
			case '@':
				$this->symbol = 'PUB_TYPE';
				$pub_type = '';
				$this->read_input();
				while ($this->input_symbol == 'c')
				{
					$pub_type .= $this->input_char;
					$this->read_input();
				}
				$pub_type = strtolower($pub_type);
				if (!in_array($pub_type, pub_types()))
					throw new Exception("Publication type `$pub_type` is not defined.");
				return new LexElem($this->symbol, $pub_type);
			case 'c':
				$str = $this->input_char;
				$this->read_input();
				while ($this->input_symbol != 's' && $this->input_symbol != '=' && $this->input_symbol != ',')
				{
					$str .= $this->input_char;
              		$this->read_input();
				}
				if (in_array(strtolower($str), bibtex_fields()))
					$this->symbol = 'KEYWORD';
				else
					$this->symbol = 'ABB';
				return new LexElem($this->symbol, $str);
			case 'n':
				$number = $this->input_char;
				$this->read_input();
				while ($this->input_symbol == 'n')
				{
					$number .= $this->input_char;
              		$this->read_input();
				}
				$this->symbol = 'NUMBER';
				return new LexElem($this->symbol, $number);
			case '"':
				$this->symbol = 'STRVAL';
				$this->read_input();
				while ($this->input_symbol != '"')
				{
					if ($this->input_char == '{') //curly bracket
					{
						$lpar_count = 1; $rpar_count = 0;
						$this->symbol = 'STRVAL';
						while ($lpar_count != $rpar_count)
						{
							$this->read_input();
							if ($this->input_char == '{')
								$lpar_count++;
							elseif ($this->input_char == '}')
								$rpar_count++;
							else
								$strval .= $this->input_char; //if none of '{' or ')' was read, we can consider this char
						}
					}
					else
						$strval .= $this->input_char;
              		$this->read_input();
				}
				$this->read_input();
				return new LexElem($this->symbol, $strval);
			case '{': //curly bracket
				$lpar_count = 1;
				$rpar_count = 0;
				$this->symbol = 'STRVAL';
				while ($lpar_count != $rpar_count)
				{
					$this->read_input();
					if ($this->input_char == '{')
						$lpar_count++;
					elseif ($this->input_char == '}')
						$rpar_count++;
					else
						$strval .= $this->input_char; //if none of '{' or ')' was read, we can consider this char
				}
				$this->read_input();
				return new LexElem($this->symbol, $strval);
			case '(':
				$this->symbol = 'LPAR';
				$this->read_input();
				return new LexElem($this->symbol, null);
			case ')':
				$this->symbol = 'RPAR';
				$this->read_input();
				return new LexElem($this->symbol, null);
			case ',':
				$this->symbol = 'COMMA';
				$this->read_input();
				return new LexElem($this->symbol, null);
			case '=':
				$this->symbol = 'ASSIGN';
				$this->read_input();
				return new LexElem($this->symbol, null);
			default:
				throw new Exception("Line #".$this->line_number.":Unexpected input char `$this->input_char`");
        }
    }
}
