<?php
session_start();
function __autoload($class_name)
{
	require_once "../common/$class_name.inc.php";
}
require_once('../common/common.inc.php');
require_once('../libs/smarty/Smarty.class.php');
require_once('../common/db_fns.inc.php');
require_once('../common/misc_fns.inc.php');
require_once('../common/SLException.inc.php');


/**
 * Generates 3 inquiry forms. Leads the submitter to add a new publication.
 *
 */
class Load
{
	private $db;
	private $smarty;
	private $request;
	private $auth_module;
	private $permitted_extensions;

	/*	Uses sessions:
		$_SESSION['feedback_st1'] - saved form data of load_st1.tpl template
		$_SESSION['feedback_st2'] - saved form data of load_st2.tpl template
		$_SESSION['feedback_st3'] - saved form data of load_st3.tpl template
		$_SESSION['tmp_attr'] - new extra attributes temporarily added by current user
		$_SESSION['tmp_cat'] - temporarily added categories
	*/

	/**
	 * Constructor
	 *
	 * @param array $request  usually $_POST
	 * @param int $permitted_extensions  permitted file extensions
	 */
	public function __construct($request, $permitted_extensions)
	{
		$this->permitted_extensions = $permitted_extensions;
		$this->request = $request;
		$this->db = db_connect();
		$this->auth_module = new AuthModule($this->db);

		$this->smarty = new Smarty;
		$this->smarty->config_dir = '../configs/';
		$this->smarty->template_dir = '../templates/';
		$this->smarty->compile_dir = '../templates_c/';
		//$this->smarty->compile_check = true;
		//$this->smarty->debugging = true;
		$this->smarty->assign('logged_info', $this->auth_module->get_login());
		global $account_url;
		$this->smarty->assign('account_url', $account_url);
		global $app_path;
		$this->smarty->assign('app_path', $app_path);
		//if no user logged in, force authorization
		if (!is_numeric($this->auth_module->get_id()))
		{
			$this->smarty->assign('refresh', "1 ; URL=".$account_url."index.php?ref=/load/");
			$this->smarty->assign('operation_error', 'To this interface you must <a href="'.$account_url.'index.php?ref=/load/">log in</a>.');
			$this->smarty->display('user_login.tpl');
			die();
		}
		if ($this->auth_module->get_level() > 1)
		{
			throw new SIException("Tou don't have rights to import publication.");
		}
	}

	/**
	 * Execute page, the only entry point of Load class
	 *
	 */
	public function execute()
	{
		$request = $this->request;
		$this->smarty->assign('level', $this->auth_module->get_level());
		if ($request['delete_author']) //delete created user
		{
			try
			{
				$_SESSION['feedback_st2'] = $this->request; //save typed form data
			 	$a = $request['authors'];
				if (sizeof($a) != 0)
				{
					$affected = 0;
					foreach ($a as $i => $value)
						if (!$affected) $affected = $this->delete_author($value);
						else $this->delete_author($value);
				}

				if ($affected)
				{
					$this->smarty->assign("operation_success", "Authors have been deleted.");
				}
				else if (sizeof($a) == 0)
				{
					$this->smarty->assign("operation_error", "No author has been chosen.");
				}
				else
				{
					$this->smarty->assign("operation_error", "The record can't be deleted, it's used by some publications.");
				}
				$this->display_st2();
			} catch (Exception $e) {
				$this->smarty->assign("operation_error", $e->getMessage());
				$this->display_st2();
			}
		} else if ($request['add_author']) {
			try
			{
				$_SESSION['feedback_st2'] = $this->request; //save typed form data
				add_author($this->db, $request['author_name'], $request['author_middle_name'], $request['author_surname'], $this->auth_module->get_id());
				$this->smarty->assign("operation_success", "Author has been added");
				$this->display_st2();
			}
			catch (SLException $s)
			{
				die($s->getMessage());
			}
			catch (Exception $e)
			{
				$this->smarty->assign("operation_error", $e->getMessage());
				$this->display_st2();
			}
		}
		else if ($request['select_author']) //choose author, copy him to the right listbox
		{
			try {
				$_SESSION['feedback_st2'] = $this->request; //save typed form data
			 	$a = $request['authors'];
			 	if (is_array(($a)))
			 	{
				 	foreach ($a as $i => $value)
				 	{
				 		if (sizeof($_SESSION["selected_authors"]) == 0)
				 			$_SESSION["selected_authors"] = array($value);
				 		else
				 			$_SESSION["selected_authors"] = array_merge($_SESSION["selected_authors"], array($value));
				 	}
				 	$_SESSION["selected_authors"] = array_unique($_SESSION["selected_authors"]);
			 	}
			 	$this->display_st2();
			} catch (Exception $e) {
				$this->smarty->assign("operation_error", $e->getMessage());
				$this->display_st2();
			}
		}
		else if ($request['deselect_author']) //remove author from selection
		{
			try {
				$_SESSION['feedback_st2'] = $this->request; //save typed form data
			 	$a = $request['selected_authors'];
			 	if (is_array($a))
			 	{
				 	foreach ($a as $value)
				 	{
				 		foreach ($_SESSION["selected_authors"] as $i => $v)
				 			if ($value == $v)
				 				$_SESSION["selected_authors"][$i] = null;
				 	}
				 	$_SESSION["selected_authors"] = $this->array_shrink($_SESSION["selected_authors"]);
			 	}
			 	$this->display_st2();
			} catch (Exception $e) {
				$this->smarty->assign("operation_error", $e->getMessage());
				$this->display_st2();
			}
		}
		else if ($request['delete_publisher'])
		{
			try {
				$_SESSION['feedback_st2'] = $this->request; //save typed form data
				$affected = $this->delete_publisher($request['publisher']);
				if ($affected)
					$this->smarty->assign("operation_success", "Publisher has been deleted.");
				else {
					$this->smarty->assign("operation_error", "You don't have rights to delete this record.");
				}
				$this->display_st2();
			} catch (Exception $e) {
				$this->smarty->assign("operation_error", $e->getMessage());
				$this->display_st2();
			}
		}
		else if ($request['delete_journal'])
		{
			try {
				$_SESSION['feedback_st2'] = $this->request; //save typed form data
				$affected = $this->delete_journal($request['journal']);
				if ($affected)
					$this->smarty->assign("operation_success", "Journal has been deleted.");
				else {
					$this->smarty->assign("operation_error", "The record can't be deleted, it's used by some publications.");
				}
				$this->display_st2();
			} catch (Exception $e) {
				$this->smarty->assign("operation_error", $e->getMessage());
				$this->display_st2();
			}
		}
		else if ($request['delete_conference'])
		{
			try {
				$_SESSION['feedback_st2'] = $this->request; //save typed form data
				$affected = $this->delete_conference($request['conference']);
				if ($affected)
					$this->smarty->assign("operation_success", "Conference has been deleted.");
				else {
					$this->smarty->assign("operation_error", "The record can't be deleted, it's used by some publications.");
				}
				$this->display_st2();
			} catch (Exception $e) {
				$this->smarty->assign("operation_error", $e->getMessage());
				$this->display_st2();
			}
		}
		else if ($request['delete_added_categories'])
		{
			try {
				$_SESSION['feedback_st1'] = $this->request; //save typed form data
				unset($_SESSION['tmp_cat']);
				$this->smarty->assign("operation_success", "Added categories have been deleted.");
				$this->display_st1();
			} catch (SLException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign("operation_error", $e->getMessage());
				$this->display_st1();
			}
		}
		elseif ($request['add_publisher'])
		{
			try {
				$_SESSION['feedback_st2'] = $this->request; //save typed form data
				add_publisher($this->db, $request['publisher_name'], $request['publisher_address'], $this->auth_module->get_id());
				$this->smarty->assign('operation_success', 'Publisher has been added.');
				$this->display_st2();
			} catch (SLException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
				$this->display_st2();
			}
		}
		elseif ($request['add_journal'])
		{
			try {
				$_SESSION['feedback_st2'] = $this->request; //save typed form data
				add_journal($this->db, $request['journal_name'], $request['journal_issn'], $this->auth_module->get_id());
				$this->smarty->assign('operation_success', 'Journal has been added.');
				$this->display_st2();
			} catch (SLException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
				$this->display_st2();
			}
		}
		elseif ($request['add_conference'])
		{
			try {
				$_SESSION['feedback_st2'] = $this->request; //save typed form data
				extract($request);
				add_conference($this->db, $conference_name, $conference_abbreviation, $w_year, $w_from, $w_to, $location, $isbn, $this->auth_module->get_id());
				$this->smarty->assign('operation_success', 'Conference has been added.');
				unset ($_SESSION['feedback_st2']['conference_name']);
				unset ($_SESSION['feedback_st2']['conference_abbreviation']);
				unset ($_SESSION['feedback_st2']['w_year']);
				unset ($_SESSION['feedback_st2']['w_from']);
				unset ($_SESSION['feedback_st2']['w_to']);
				unset ($_SESSION['feedback_st2']['isbn']);
				$this->display_st2();
			} catch (SLException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
				$this->display_st2();
			}
		}
		else if ($request['add_category'])
		{
			try {
				$_SESSION['feedback_st1'] = $this->request; //save typed form data
				add_tmp_category($this->db, $request['category_name']);
				$this->smarty->assign('operation_success', 'Category has been added.');
				$this->display_st1();
			} catch (SLException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
				$this->display_st1();
			}
		}
		else if ($request['add_attribute'])
		{
			$_SESSION['feedback_st2'] = $this->request; //save typed form data
			try {
				$this->validate_add_att();
				$this->add_tmp_attribute($request['new_attribute_name'], $request['new_attribute_description'], $request['new_attribute_value']);
				$this->display_st2();
			} catch (SLException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
				$this->display_st2();
			}
		}
		else if ($request['update_publisher'])
		{
			try {
				update_publisher($this->db, $this->request['publisher'], $this->request['publisher_name'], $this->request['publisher_address'], $this->auth_module->get_id());
				$this->smarty->assign('operation_success', 'Publisher has been updated.');
				$this->display_st2();
			} catch (Exception $e) {
				$this->smarty->assign('operation_error', $e->getMessage());
				$this->display_st2();
			}
		}
		else if ($request['update_journal'])
		{
			try {
				update_journal($this->db, $this->request['journal'], $this->request['journal_name'], $this->request['journal_issn'], $this->auth_module->get_id());
				$this->smarty->assign('operation_success', 'Journal has been updated');
				$this->display_st2();
			} catch (Exception $e) {
				$this->smarty->assign("operation_error", $e->getMessage());
				$this->display_st2();
			}
		}
		elseif ($request['update_conference'])
		{
			try {
				extract($this->request);
				update_conference($this->db, $conference, $conference_name, $conference_abbreviation, $w_year, $w_from, $w_to, $location, $isbn, $this->auth_module->get_id());
				$this->smarty->assign('operation_success', 'Conference has been updated');
				$this->display_st2();
			} catch (Exception $e) {
				$this->smarty->assign("operation_error", $e->getMessage());
				$this->display_st2();
			}
		}
		else if ($request['delete_attributes'])
		{
			$_SESSION['feedback_st2'] = $this->request; //save typed form data
			try {
				$this->delete_attributes();
				$this->display_st2();
			} catch (SLException $s) {
				die($s->getMessage());
			} catch (Exception $e) {
				die($e->getMessage());
			}
		}
		else if ($request['previous']) {
			switch ($request['prev_stage']) {
				case(1):
					$_SESSION['feedback_st2'] = $this->request; //save typed form data
					$this->display_st1();
					break;
				case(2):
					$this->display_st2();
					break;
				case(3):
					$this->display_st3();
					break;
				default:
					$this->display_st1();
			}
		}
		else if (!$request || $request['next']) {
			switch ($request['next_stage']) {
				case(''): //start stage
					$this->display_st1();
					break;
				case(2): //additional attributes
					try {
						$this->validate_st1();
						$this->display_st2();
					} catch (SLException $s) {
						die($s->getMessage());
					} catch (Exception $e) {
						$this->smarty->assign("operation_error", $e->getMessage());
						$this->display_st1();
					}
					break;
				case(3): //confirm stage
					try {
						$validation_report = $this->validate_st2();
						$this->display_st3($validation_report);
					} catch (SLException $s) {
						die($s->getMessage());
					} catch(Exception $e) {
						$this->smarty->assign("operation_error", $e->getMessage());
						$this->display_st2();
					}
					break;
				case(4): //finish stage
					try {
						$this->validate_st3();
						$this->finish();
					} catch (SLException $s) {
						die($s->getMessage());
					} catch(Exception $e) {
						$this->smarty->assign("operation_error", $e->getMessage());
						$this->display_st3();
					}
					break;
				default:
					$this->display_st1();
			}
		}
		else {
       		$this->display_st1();
	    }
	}

	/**
	 * Removes null values from an array and shriks it's content
	 *
	 * @param array $arr
	 */
	private function array_shrink($arr) {
		foreach ($arr as $i => $v)
		{
			if ($v != null)
				$ret[$i] = $v;
		}
		return $ret;
	}


	/**
	 * Delete author from temporary list
	 *
	 * @param int $author_id id of author
	 */
	private function delete_author_from_temp($author_id)
	{
		$_SESSION['selected_authors'] = array_diff($_SESSION['selected_authors'], array($author_id));
	}


	/**
	 * Delete author - deletion will succeed only if author is
	 * not binded to a publication.
	 *
	 * @param int $id
	 * @return int count of affected rows
	 */
	private function delete_author($id)
	{
		$affected = delete_author($this->db, $id);
		unset($_SESSION['feedback_st1']['authors']);
		if ($affected == 1)
			$this->delete_author_from_temp($id);
		return $affected;
	}

	/**
	 * Delete unbinded publisher
	 *
	 * @param int $id publisher id
	 * @return int count of affected rows
	 */
    private function delete_publisher($id)
    {
		$affected = delete_publisher($this->db, $id);
		unset($_SESSION['feedback_st2']['publisher']);
		return $affected;
	}

	/**
	 * Delete unbinded journal
	 *
	 * @param int $id journal id
	 * @return int count of affected rows
	 */
    private function delete_journal($id)
    {
		$affected = delete_journal($this->db, $id);
		unset($_SESSION['feedback_st2']['journal']);
		return $affected;
	}

	/**
	 * Delete unbinded conference
	 *
	 * @param int $id conference id
	 * @return int count of affected rows
	 */
    private function delete_conference($id)
    {
		$affected = delete_conference($this->db, $id);
		unset($_SESSION['feedback_st2']['conference']);
		return $affected;
	}

	/**
	 * Delete temporary attribute from sessions
	 *
	 *
	 */
	private function delete_attributes()
	{
		unset($_SESSION['tmp_attr']);
	}

	/**
	 * Creates a temporary information about new attributes
	 *
	 * @param string $name  name of the new added attribute
	 * @param string $description  description of the new added attribute
	 * @param string $value  text value of the attribute
	 */
	private function add_tmp_attribute($name, $description, $value)
	{
		$_SESSION['tmp_attr']['name'][$name] = $value;
		$_SESSION['tmp_attr']['description'][$name] = $description;
	}

	/**
	 * Add new attribute to the publication, assign a value to it
	 *
	 * @param int $publication_id  id of the publication to which this attribute is beeing assigned
	 * @param string $name  name of the new added attribute
	 * @param string $description  the name of the attribute
	 * @param string $value  text value of the attribute
	 */
	private function add_attribute($publication_id, $name, $description, $value)
	{
		if (!$description || !$value)
			throw new Exception('Missing information about new attribute.');
		$this->db->autoCommit(false);
		$submitter_id = $this->auth_module->get_id();
		$sql = "INSERT INTO `attributes` VALUES ('', '$submitter_id', '$name', '$description', '0')";
		$res1 =& $this->db->query($sql);
		if (DB::isError($res1)) {
			$this->db->rollBack();
			throw new Exception($res1->getMessage());
		}
		$attributes_id = last_insert_id($this->db);
		$sql = "INSERT INTO `attrib_storage` VALUES ('$publication_id', '$attributes_id', '$submitter_id', '$value')";
		$res2 = $this->db->query($sql);
		if (DB::isError($res2)) {
			$this->db->rollBack();
			throw new SLException($res2->getMessage());
		}
		$this->db->commit();
	}

	/**
	 * Display the 1. stage of the forms
	 *
	 */
	private function display_st1()
	{
		$sql = "SELECT id, name
				FROM `categories`
				ORDER BY name";
		$this->smarty->assign('categories', htmlspecialchars_rec($this->db->getAssoc($sql)));

		$this->smarty->assign("pub_types", pub_types_full());
		$s=null;
		if (is_array($_SESSION['tmp_cat'])) foreach($_SESSION['tmp_cat'] as $name)
		{
			$s[] = $name;
		}
		$this->smarty->assign('added_categories', stripslashes_rec(htmlspecialchars_rec($s)));
		if (isset($_SESSION['feedback_st1'])) $this->feedback_st1(); //feedback
		$this->smarty->display('load_st1.tpl');
	}

	/**
	 * Display the 2. stage of the forms
	 *
	 */
	private function display_st2()
	{
		if (isset($_SESSION['feedback_st2'])) $this->feedback_st2(); //feedback

		$sql = "SELECT id, CONCAT_WS(' ',name,middlename,surname) AS author
				FROM `author`
				ORDER BY surname, name";
		$arr = $this->db->getAssoc($sql);
		if (PEAR::isError($arr))
			throw new SLException($arr->getMessage());
		$this->smarty->assign('authors', htmlspecialchars_rec($arr));

		$sql = "SELECT id, name
				FROM `publisher`
				ORDER BY name";
		$arr = $this->db->getAssoc($sql);
		if (PEAR::isError($arr))
			throw new SLException($arr->getMessage());
		$this->smarty->assign('publisher', htmlspecialchars_rec($arr));

		$sql = "SELECT id, name
				FROM `journal`
				ORDER BY name";
		$arr = $this->db->getAssoc($sql);
		if (PEAR::isError($arr))
			throw new SLException($arr->getMessage());
		$this->smarty->assign('journal', htmlspecialchars_rec($arr));

		//select either complete conferences
		//(with name and year) or select non-complete conference only to the user owning the record
		$sql = "SELECT id, name
				FROM `conference`
				WHERE w_year<>'0000' OR (w_year='0000' AND submitter_id='".$this->auth_module->get_id()."')
				ORDER BY name";
		$arr = $this->db->getAssoc($sql);
		if (PEAR::isError($arr))
			throw new SLException($arr->getMessage());
		$this->smarty->assign('conference', htmlspecialchars_rec($arr));

		$this->smarty->assign("dpub_type", $_SESSION['feedback_st1']['pub_type']);
		$sql = "SELECT id, name FROM attributes WHERE confirmed = '1'";
		$arr =& $this->db->getAssoc($sql);
		$sql = "SELECT id, description FROM attributes WHERE confirmed = '1'";
		$desc =& $this->db->getAssoc($sql);
		$out = '';
		foreach ($arr as $i => $v)
		{
			$out .= '<tr><td>'.$v.($desc[$i]?'<br />('.$desc[$i].')':'').'</td><td><input type="text" name="'.$v.'" size="20" value="'.stripslashes(htmlspecialchars($_SESSION['feedback_st2'][$v])).'" /></td></tr>'; //this row includes feedback operation
		}
		$this->smarty->assign("attributes", $out);	//insert form fields with additional attributes
		$s = '';
		$tmp = $_SESSION['tmp_attr']['name']; //$tmp is an array
		if (is_array($tmp))
		{
			foreach ($tmp as $i => $value)
			{
				$description = $_SESSION['tmp_attr']['description'][$i];
				$s .= '<tr><td colspan="2">you are adding these attributes:</td></tr>
				<tr><td><strong>'.$i.'</strong>'.($description?'<br />('.stripslashes(htmlspecialchars($description)).')':'').'</td><td><span class="tmp_att_value">'.stripslashes(htmlspecialchars($value)).'</span></td></tr>';
			}
		}
		$this->smarty->assign('tmp_attr', $s);
		$this->smarty->display('load_st2.tpl');
	}

	/**
	 * Display the 3. stage of the forms - final overview and file upload
	 *
	 * @param string validation report from the previous form
	 */
	private function display_st3($report)
	{
		$this->smarty->assign('report', $report);
		$this->smarty->assign("logged_info", $this->auth_module->get_login());

		$data = array_merge($_SESSION['feedback_st1'], $_SESSION['feedback_st2']);
		$data = stripslashes_rec(htmlspecialchars_rec($data));

		//user data from text inputs
		foreach ($data as $i => $v)
			$this->smarty->assign($i, $v);

		//authors
		$authors = $_SESSION['selected_authors'];
		$condition = ''; $i = 0; $authors_tog = '';
		if (is_array($authors))
		{
			$count = count($authors);
			foreach ($authors as $id) {
				$sql = "SELECT CONCAT_WS(' ',name,middlename,surname) AS author
						FROM `author`
                		WHERE id = '$id'";
				$authors_tog .= $this->db->getOne($sql);
				if (++$i < $count) $condition .= $authors_tog .= ', ';
			}
		}

		$this->smarty->assign('authors', htmlspecialchars_rec($authors_tog)); //associative authors

		//publisher
		$row_publisher = get_publisher_data($this->db, $data['publisher']);
    	$this->smarty->assign('publisher_data', htmlspecialchars_rec($row_publisher)); //publisher name
        //conference
		$row_conference = get_conference_data($this->db, $data['conference']);
		$this->smarty->assign('conference_data', htmlspecialchars_rec($row_conference));
        //journal
        $row_journal = get_journal_data($this->db, $data['journal']);
		$this->smarty->assign('journal_data', htmlspecialchars_rec($row_journal));
		//checked categories
		$s = null;
		$propagated_categories = $_SESSION['feedback_st1']['propagated_categories'];
		if (is_array($propagated_categories))
			foreach ($propagated_categories as $i => $v) {
				$s[] = $this->db->getOne("SELECT name FROM `categories` WHERE id='$v'");
		}
		$this->smarty->assign('categories', stripslashes_rec(htmlspecialchars_rec($s)));
		//added categories
		$s = null; $tmp = $_SESSION['tmp_cat'];
		if (is_array($tmp)) {
			foreach ($tmp as $i => $v) {
				$s[] = $v;
			}
		}
		$this->smarty->assign('tmp_cat', stripslashes_rec(htmlspecialchars_rec($s)));
		//filled public attributes
		$s = null;
		$ex_attr = $this->filled_ext_attr();
		if (is_array($ex_attr))
			foreach ($ex_attr as $name) {
				$s .= stripslashes(htmlspecialchars($name)).': '.stripslashes(htmlspecialchars($_SESSION['feedback_st2'][$name])).'<br />';
		}
		$this->smarty->assign('attributes', $s);
		//added tmp attributes
		$s = null; $tmp = $_SESSION['tmp_attr']['name']; //$tmp is an array
		if (is_array($tmp)) {
			foreach ($tmp as $i => $v) {
				$s .= stripslashes(htmlspecialchars($i)).': '.stripslashes(htmlspecialchars($v)).'<br />';
			}
		}
		$this->smarty->assign('tmp_attr', $s);
		$out='';
		$permitted_extensions = $this->permitted_extensions;
		$count=count($permitted_extensions)-1;
		if ($permitted_extensions) foreach($permitted_extensions as $v) {
			$out .= $v.($count--?', ':''); //insert commas only between numbers
		}
		$this->smarty->assign('permitted_extensions', $out);
		$this->smarty->display('load_st3.tpl');
	}

	/**
	 * Gives written data back to form stage 1
	 *
	 */
	private function feedback_st1()
	{
		$feedback = $_SESSION['feedback_st1'];
		$feedback = stripslashes_rec($feedback);
		$this->smarty->assign('title', $feedback['title']);
		$this->smarty->assign('selected_pub_type', $feedback['pub_type']);
		$this->smarty->assign('abstract', $feedback['abstract']);
		$this->smarty->assign('selected_categories', $feedback['propagated_categories']);
	}

	/**
	 * Gives written data back to form stage 2
	 *
	 */
    private function feedback_st2()
    {
		$feedback = $_SESSION['feedback_st2'];
		$feedback = htmlspecialchars_rec($feedback);
		foreach ($feedback as $i => $v)
		{
			if ($i != 'authors' && $i != 'journal' && $i != 'conference' && $i != 'publisher')
				$this->smarty->assign($i, $v);
		}

		//authors - select options
		$author_names = array();
		try
		{
			if (sizeof($_SESSION["selected_authors"]) != 0)
				foreach ($_SESSION["selected_authors"] as $id)
			{
				$author_name = $this->db->getOne("select CONCAT_WS(' ',name,middlename,surname) AS author from author where id=?", $id);
				if (PEAR::isError($author_name))
		 	   		throw new Exception($author_name->getMessage(), 1);
		 		$author_names = array_merge($author_names, array($author_name));
			}
		}
		catch (Exception $e)
		{
			throw new SLException($e->getMessage());
		}
		$this->smarty->assign("selected_authors", $_SESSION["selected_authors"]);
		$this->smarty->assign("selected_authors_names", $author_names);

		//other select options
		$this->smarty->assign('selected_journal', $feedback['journal']);
		$this->smarty->assign('selected_conference', $feedback['conference']);
		$this->smarty->assign('selected_publisher', $feedback['publisher']);
	}

	/**
	 * Save form 1 data to $_SESSION, check if typed data were valid
	 *
	 */
	private function validate_st1()
	{
		$_SESSION['feedback_st1'] = $this->request; //save typed form data
		extract($this->request);
		if (!in_array($pub_type, pub_types()))
			throw new Exception('Type of publication is not selected.');
		if (!$title)
			throw new Exception('Title of publication is missing.');
	}

	/**
	 * Save form 2 data to $_SESSION and check if typed data are valid
	 * using bibtex rules. If some problem is recognized, returns report.
	 *
	 *	@return string validation report
	 */
	private function validate_st2()
	{
		$_SESSION['feedback_st2'] = $this->request; //save typed form data
		$bib = Bibtex::create($_SESSION['feedback_st1']['pub_type']);
		$data = array_merge($_SESSION['feedback_st1'], $_SESSION['feedback_st2']);
		$data = array_merge($data, array('author'=>$_SESSION['selected_authors'])); //preserve bibtex fieldnames

		if (is_array($d = get_publisher_data($this->db, $this->request['publisher'])))
			$data = array_merge($data, $d);
		if (is_array($d = get_journal_data($this->db, $this->request['journal'])))
			$data = array_merge($data, $d);
		if (is_array($d = get_conference_data($this->db, $this->request['conference'])))
		{
			$ym = array('year' => $d['year'], 'month' => substr($data['w_from'], 5, 2));//preserve bibtex fieldnames
			$data = array_merge($data, $d, $ym);
		}
		else
		{
			$ym = array('year' => $data['issue_date'], 'month' => substr($data['issue_date'], 5, 2));//preserve bibtex fieldnames
			$data = array_merge($data, $ym);
		}
		$report = $bib->validate($data);
		return $report;
	}

	/**
	 * Checks whether submitter has sent a pdf/ppt or not
	 *
	 */
	private function validate_st3()
	{
		$permitted_extensions = $this->permitted_extensions;
		foreach($_FILES['documents']["error"] as $key => $error)
		{
			if ($error == UPLOAD_ERR_OK)
			{
				$file =& $_FILES['documents']['tmp_name'][$key];
				$name =& $_FILES['documents']['name'][$key];
				$extension = substr($name, strrpos($name, ".") + 1);
				if ($permitted_extensions)
				{
					if (!in_array($extension, $permitted_extensions))
						throw new Exception("You are adding file with a non-permited extension.");
				}
				if ((!is_uploaded_file($file) && $file))
					throw new SLException('Not an uploaded file.');
			}
		}
	}

	/**
	 * Validate temporarily added extensible attributes
	 *
	 */
	private function validate_add_att(){
		$_SESSION['feedback_st2'] = $this->request; //save typed form data
		extract($this->request);
		if (!ereg('^_([0-9]|[a-zA-Z]){0,31}$', $new_attribute_name))
			throw new Exception("First char chould be '_', name should contain letters of alphabet and numbers, max. length is 32.");
		if (!$new_attribute_value)
			throw new Exception('Attribute value is missing');
		if (sizeof($new_attribute_description) > 255)
			throw new Exception('Attribute description is too long.');
		unset($_SESSION['feedback_st2']['new_attribute_name']);
		unset($_SESSION['feedback_st2']['new_attribute_value']);
		unset($_SESSION['feedback_st2']['new_attribute_description']);
	}

	/**
	 * Recognize which extensible attribute in form stage 2 have been assigned
	 *
	 * @return array of $attribute_id => $attrinute_name
	 */
	private function filled_ext_attr() {
		foreach ($_SESSION['feedback_st2'] as $name => $value) {
			$sth = $this->db->prepare("SELECT id, name FROM `attributes` WHERE name=? AND confirmed='1'");
			if (PEAR::isError($sth)) {
				$this->db->rollBack();
				throw new SLException($sth->getMessage());
			}
			$res =& $this->db->execute($sth, $name);
			if ($res->numRows()) {
				$row = $res->fetchRow();
				$id = $row[0]; $nm = $row[1];
				if ($_SESSION['feedback_st2'][$nm] != '') //if value of the extensible attribute in form stage 2 had been assigned
					$public_attr[$id] = $nm;
			}
		}
		return $public_attr;
	}

	/**
	 * Finish adding new publication to the database, save pdf/ppt
	 *
	 */
	private function finish()
	{
		$submitter_id = $this->auth_module->get_id();
		$data = array_merge($_SESSION['feedback_st1'], $_SESSION['feedback_st2']);
		$publisher_id = $data['publisher'];
		$conference_id = $data['conference'];
		$journal_id = $data['journal'];
		$authors = $_SESSION['selected_authors'];
		$propagated_categories = $_SESSION['feedback_st1']['propagated_categories'];
		extract($data);

		if (!title)
			throw new Exception("Can't be inserted publication without a title.");

		$this->db->autoCommit(false);
		//validation - checking if counters still exist
		$val_report = validate_counters($this->db, $authors, $propagated_categories, $publisher_id, $journal_id, $conference_id);
		if ($val_report)
		{
			$this->db->rollback();
			$this->smarty->assign('vcreport', nl2br($val_report));
			$this->smarty->display('load_stFinish.tpl');
			die;
		}

		//insert into `publication`
		$sql = "INSERT INTO `publication`
				(id, submitter_id, publisher_id, conference_id, journal_id, pub_type, title, booktitle, abstract, issue_date, isbn, volume, number, pages, note, chapter, edition, editor, howpublished, institution, organization, school, address, type_of_report, url, confirmed) VALUES
				('', '$submitter_id', '$publisher_id', '$conference_id', '$journal_id', '$pub_type', '$title', '$booktitle', '$abstract', '$issue_date', '$isbn', '$volume', '$number', '$pages', '$note', '$chapter', '$edition', '$editor', '$howpublished', '$institution', '$organization', '$school', '$address', '$type', '$url', 0)";
		$res =& $this->db->query($sql);
		if (DB::isError($res)) {
			$this->db->rollBack();
			throw new SLException($res->getMessage());
		}
		$publication_id = last_insert_id($this->db);
		//insert into `author_has_publication`
		if (sizeof($authors) != 0)
		{
			$sth = $this->db->prepare("INSERT INTO `author_has_publication` VALUES (?, ?, ?)");
			if (PEAR::isError($sth))
			{
				$this->db->rollBack();
				throw new SLException($sth->getMessage());
			}
			$priority = 0;
			foreach ($authors as $_id)
			{
				$res =& $this->db->execute($sth, array($_id, $publication_id, $priority++));
				if (PEAR::isError($res))
				{
					$this->db->rollBack();
					throw new SLException($res->getMessage());
				}
			}
		}
		//insert into `categories`
		$inserted_cat_ids = null;
		$tmp = $_SESSION['tmp_cat'];
		if (is_array($tmp)) {
			$sth =& $this->db->prepare("INSERT INTO `categories` VALUES('', '$submitter_id', ?)");
			if (PEAR::isError($sth)) {
				$this->db->rollBack();
				throw new SLException($sth->getMessage());
			}
			foreach ($tmp as $name) {
				$res =& $this->db->execute($sth, stripslashes($name));//stripslashes because prepare-execute add slashes redundantly
				$inserted_cat_ids[] = last_insert_id($this->db);
				if (DB::isError($res)) {
					$this->db->rollBack();
					throw new SLException($res->getMessage());
				}
			}
		}
		//insert into `categories_has_publication`
		$tmp = $_SESSION['tmp_cat'];
		$sth =& $this->db->prepare("INSERT INTO `categories_has_publication` VALUES(?, ?)");
		if (PEAR::isError($sth)) {
			$this->db->rollBack();
			throw new SLException($sth->getMessage());
		}
		if (is_array($inserted_cat_ids)) {
			foreach ($inserted_cat_ids as $category_id) {
				$res =& $this->db->execute($sth, array($publication_id, $category_id));
				if (DB::isError($res)) {
					$this->db->rollBack();
					throw new SLException($res->getMessage());
				}
			}
		}
		if (is_array($propagated_categories)) {
			foreach ($propagated_categories as $category_id) {
				$res =& $this->db->execute($sth, array($publication_id, $category_id));
				if (DB::isError($res)) {
					$this->db->rollBack();
					throw new SLException($res->getMessage());
				}
			}
		}
		//insert public attributes
		$public_attr = $this->filled_ext_attr();
		if (is_array($public_attr)) { //prepare attribute insertion
			$sth = $this->db->prepare("INSERT INTO `attrib_storage` VALUES ('$publication_id', ?, '$submitter_id', ?)");
			if (PEAR::isError($sth)) {
				$this->db->rollBack();
				throw new SLException($sth->getMessage());
			}
			foreach ($public_attr as $id => $name) {
				//insert into `attrib_storage`
				$res1 =& $this->db->execute($sth, array($id, stripslashes($_SESSION['feedback_st2'][$name])));//stripslashes because prepare-execute add slashes redundantly
				if (DB::isError($res1)) {
					$this->db->rollBack();
					throw new SLException($res1->getMessage());
				}
			}
		}
		//----
		$attributes = $_SESSION['tmp_attr']['name'];
		if(is_array($attributes)) foreach ($attributes as $name => $value) {
			$description = $_SESSION['tmp_attr']['description'][$name]; //attribute description
			//insert into `attributes
			$sql = "INSERT INTO `attributes` VALUES ('', '$submitter_id', '$name', '$description', '0')";
			$res =& $this->db->query($sql);
			if (DB::isError($res)) {
				$this->db->rollBack();
				throw new SLException($res->getMessage());
			}
			$attributes_id = last_insert_id($this->db);
			//insert into `attrib_storage`
			$sql = "INSERT INTO `attrib_storage` VALUES ('$publication_id', '$attributes_id', '$submitter_id', '$value')";
			$res =& $this->db->query($sql);
			if (DB::isError($res)) {
				$this->db->rollBack();
				throw new SLException($res->getMessage());
			}
		}
		//store pdf/ppt into the filesystem
		save_documents($this->db, $publication_id, $title);

		$this->db->commit();

		$_SESSION['feedback_st1'] = $_SESSION['feedback_st2'] = $_SESSION['tmp_cat'] = null;
		$this->smarty->assign('logged_info', $this->auth_module->get_login());
		$this->smarty->display('load_stFinish.tpl');
	}
}
?>